// StartConsole.cs
// 
// Copyright (C) 2008,2009,2010 Bartek Jasicki
//
// This file is part of Grubng.
//
//  Grubng is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//   Grubng is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with Grubng.  If not, see <http://www.gnu.org/licenses/>.
//

/**@file StartConsole.cs
 * Start console version of program.*/

using System;
using System.Diagnostics;
using System.IO;
using Mono.Unix;

namespace Grubng
{
	
	/// <summary>
	/// Start console version of program.
	/// </summary>
	internal static class StartConsole
	{
		
		/// <summary>
		/// Create console GUI for program.
		/// </summary>
		/// <param name="basedir">
		/// A <see cref="System.String"/> Full path to program directory.
		/// </param>
		public static void StartGUI(string basedir)
		{
			//Check for running instance of client (if any, terminate this and show info to user)
			string filepath = Environment.GetFolderPath(Environment.SpecialFolder.Personal) + 
					Path.DirectorySeparatorChar + ".grub" + Path.DirectorySeparatorChar + "grub.pid";
			if (File.Exists(filepath))
			{
				Console.WriteLine(Catalog.GetString("You cannot run more than one client simultaneously"));
				return;
			}
			Console.Write(Catalog.GetString("Start..."));
			bool rootneed = false;
			if (Environment.OSVersion.Platform.ToString() == "Unix")
			{
				string[] path = basedir.Split(Path.DirectorySeparatorChar);
				if (path[1] != "home")
				{
					rootneed = true;
				}
				path = null;
			}
			Config conf = new Config();
			bool checkfile = File.Exists(conf.FilePath);
			if (!checkfile)
			{
				Console.WriteLine(Catalog.GetString("Username:"));
				string username = Console.ReadLine();
				Console.WriteLine(Catalog.GetString("Password:"));
				string password = Console.ReadLine();
				conf.CreateConfig(username, password);
			}
			else
			{
				Console.Write(Catalog.GetString("Checking correctness of configuration file..."));
				bool updateconf = conf.CheckConfig();
				if (updateconf)
				{
					Console.WriteLine(Catalog.GetString("Updated"));
				}
				else
				{
					Console.WriteLine(Catalog.GetString("Done"));
				}
			}
			//Create .pid file
			MainClass.CreatePIDFile();
			long needmb = Convert.ToInt64(Config.ReadConfig("/configuration/work/maxmbdownload", conf.FilePath)) * 
					Convert.ToInt32(Config.ReadConfig("/configuration/work/crawlers", conf.FilePath));
			if (Environment.OSVersion.Platform.ToString() == "Unix")
			{
				UnixDriveInfo drive = new UnixDriveInfo(Config.ReadConfig("/configuration/work/workdir", conf.FilePath));
				if ((needmb * 1048576) > drive.AvailableFreeSpace)
				{
					Console.WriteLine(Catalog.GetString("You have less than") + " " + needmb.ToString() + " " + 
					                  Catalog.GetString("MB free disk space. Please increase it before start."));
				}
				drive = null;
			}
			string checkupdates = Config.ReadConfig("/configuration/other/checkupdates", conf.FilePath);
			if (checkupdates == "Y")
			{
				string rootpass = String.Empty;
				string unstable = Config.ReadConfig("/configuration/other/unstable", conf.FilePath);
				Console.WriteLine(Catalog.GetString("Checking for updates..."));
				System.Reflection.Assembly asm = System.Reflection.Assembly.GetExecutingAssembly();
				string version = asm.GetName().Version.Major.ToString() + "." + asm.GetName().Version.Minor.ToString() + "." + asm.GetName().Version.Build.ToString();
				Updater upd = new Updater(version);
				string update = upd.CheckUpdate(unstable);
				if ((update == "current") || (update == "older"))
				{
					Console.WriteLine(Catalog.GetString("Updating..."));
					if (rootneed)
					{
						Console.WriteLine(Catalog.GetString("Enter root password:"));
						rootpass = Console.ReadLine();
					}
					string[] updated = Updater.Update(update, unstable, rootpass);
					Console.WriteLine(Catalog.GetString("Updated files:"));
					for (int i = 0; i < updated.Length; i++)
					{
						Console.WriteLine(updated[i]);
					}
					Console.WriteLine(Catalog.GetString("Client successfully updated. ") + Catalog.GetString("You must restart it again. ") + Catalog.GetString("Press any key for exit."));
					int result = Console.Read();
					result++;
					return;
				}
				conf = null;
				if (update == "none")
				{
					Console.WriteLine(Catalog.GetString("none"));
				}
				else
				{
					Console.WriteLine(Catalog.GetString("failed"));
				}
			}
			else
			{
				Console.WriteLine(Catalog.GetString("Done"));
			}
			ConsoleMain console = new ConsoleMain();
			console.Start();
			console.Dispose();
		}
	}
}
