// AboutDialog.cs
// 
// Copyright (C) 2008,2009,2010,2011 Bartek Jasicki
//
// This file is part of Grubng.
//
// Grubng is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

/**@file AboutDialog.cs
 * Provide GTK Dialog for informations about program.
 * @fn virtual void Grubng::AboutDialog::Build()
 * Create part of GUI and show it to user*/

using System;
using System.Reflection;
using Gtk;
using Grubng;
using Mono.Unix;

namespace Grubng
{
	
	/// <summary>
	/// Provide GTK Dialog for informations about program.
	/// </summary>
	internal sealed partial class AboutDialog : Gtk.Dialog
	{
		/// <summary>
		/// GTK Button for show program license dialog.
		/// </summary>
		Button btnlicense;
		/// <summary>
		/// GTK Button for close dialog.
		/// </summary>
		Button buttonOk;
		/// <summary>
		/// GTK Button for show program contributors.
		/// </summary>
		Button btnauthors;
		
		/// <summary>
		/// Standard class constructor. Create dialog and show it to user.
		/// </summary>
		public AboutDialog()
		{
			this.Build();
			//Add button btnauthors
			this.btnauthors = new Button();
			this.btnauthors.Label = Catalog.GetString("Authors");
			this.AddActionWidget(btnauthors, 0);
			this.btnauthors.Clicked += new EventHandler(OnBtnauthorsClicked);
			this.btnauthors.ShowAll();
			//Add button btnlicense
			this.btnlicense = new Button();
			this.btnlicense.Label = Catalog.GetString("License");
			this.AddActionWidget(btnlicense, 0);
			this.btnlicense.Clicked += new EventHandler(OnBtnlicenseClicked);
			this.btnlicense.ShowAll();
			//Add button buttonOk 
			this.buttonOk = new Button();
			this.buttonOk.Image = new Image(System.IO.Path.Combine(System.AppDomain.CurrentDomain.BaseDirectory, "Images" + System.IO.Path.DirectorySeparatorChar + "ok.png"));
			this.buttonOk.ImagePosition = PositionType.Left;
			this.buttonOk.Label = Catalog.GetString("_OK");
			this.buttonOk.UseUnderline = true;
			this.AddActionWidget(buttonOk, -5);
			this.buttonOk.Clicked += new EventHandler(OnButtonOkClicked);
			this.buttonOk.CanDefault = true;
			this.buttonOk.GrabFocus();
			this.buttonOk.ShowAll();
			Assembly asm = Assembly.GetExecutingAssembly();
			object[] attributes = asm.GetCustomAttributes(typeof(AssemblyCopyrightAttribute), false);
			this.lblname.Text = asm.GetName().Name + " " + asm.GetName().Version.ToString();
			this.lblcopy.Text = ((AssemblyCopyrightAttribute)attributes[0]).Copyright;
			this.Icon = new Gdk.Pixbuf(System.IO.Path.Combine(System.AppDomain.CurrentDomain.BaseDirectory, "Images" + System.IO.Path.DirectorySeparatorChar + "grubby.ico"));
			this.image2.Pixbuf = new Gdk.Pixbuf(System.IO.Path.Combine(System.AppDomain.CurrentDomain.BaseDirectory, "Images" + System.IO.Path.DirectorySeparatorChar + "splash.jpg"));
		}

		/// <summary>
		/// Function open web browser to show home page.
		/// Raised when button btnlink clicked.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> unused.
		/// </param>
		/// <param name="e">
		/// A <see cref="System.EventArgs"/> unused.
		/// </param>
		void OnBtnlinkClicked (object sender, System.EventArgs e)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (e != null)
			{
				e = null;
			}
			System.Diagnostics.Process proc = System.Diagnostics.Process.Start("http://grub.org");
			proc.Close();
		}
		
		/// <summary>
		/// Function close dialog.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> unused.
		/// </param>
		/// <param name="e">
		/// A <see cref="System.EventArgs"/> unused.
		/// </param>
		void OnButtonOkClicked (object sender, System.EventArgs e)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (e != null)
			{
				e = null;
			}
			this.Destroy();
		}
		
		/// <summary>
		/// Show COPYING file. Raised when button License is selected.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> unused.
		/// </param>
		/// <param name="e">
		/// A <see cref="System.EventArgs"/> unused.
		/// </param>
		void OnBtnlicenseClicked (object sender, System.EventArgs e)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (e != null)
			{
				e = null;
			}
			HelpDialog help = new HelpDialog("COPYING");
			int result = help.Run();
			if (result < 0)
			{
				help.Dispose();
			}
		}
		
		/// <summary>
		/// Show AUTHORS file. Raised when button btnuthors is clicked.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> unused.
		/// </param>
		/// <param name="e">
		/// A <see cref="System.EventArgs"/> unused.
		/// </param>
		void OnBtnauthorsClicked (object sender, System.EventArgs e)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (e != null)
			{
				e = null;
			}
			HelpDialog help = new HelpDialog("AUTHORS");
			int result = help.Run();
			if (result < 0)
			{
				help.Dispose();
			}
		}
		
		/// <summary>
		/// Function override Gtk.Object.Dispose function.
		/// </summary>
		public override void Dispose()
		{
			this.vbox2.Dispose();
			this.image2.Dispose();
			this.lblname.Dispose();
			this.lblinfo.Dispose();
			this.lblcopy.Dispose();
			this.btnlink.Dispose();
			this.btnlicense.Dispose();
			this.buttonOk.Dispose();
			this.btnauthors.Dispose();
			base.Dispose();
		}
	}
}
