// CrawlerInfo.cs
// 
// Copyright (C) 2008,2009,2010,2011 Bartek Jasicki
//
// This file is part of Grubng.
//
// Grubng is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

/**@file CrawlerInfo.cs
 * Provide GTK Dialog for show informations about selected crawler.
 * @fn virtual void Grubng::CrawlerInfo::Build()
 * Create part of GUI and show it to user*/

using System;
using System.Timers;
using System.Threading;
using Gtk;
using Mono.Unix;
using Grubng;

namespace Grubng
{	
	/// <summary>
	/// Provide GTK Dialog for show informations about selected crawler.
	/// </summary>
	internal sealed partial class CrawlerInfo : Gtk.Dialog
	{		
		/// <summary>
		/// GTK Button for clear console.
		/// </summary>
		Button buttonClear;
		/// <summary>
		/// GTK Button for close window.
		/// </summary>
		Button buttonOk;
		/// <summary>
		/// Number of crawler.
		/// </summary>
		int num;
		/// <summary>
		/// Thread of crawler.
		/// </summary>
		Threads crawlerthread;
		/// <summary>
		/// Status of crawler.
		/// </summary>
		int status;
		/// <summary>
		/// Array of translated crawler statuses.
		/// </summary>
		string[] crawlerstatus;
		/// <summary>
		/// Arrays of translated crawler messages.
		/// </summary>
		string[] crawlermessage;
		
		/// <summary>
		/// Standard class constructor. Create dialog and show it to user.
		/// </summary>
		/// <param name="number">
		/// A <see cref="System.Int32"/> Number of crawler
		/// </param>
		/// <param name="thread">
		/// A <see cref="Threads"/> Crawler thread
		/// </param>
		public CrawlerInfo(int number, Threads thread)
		{
			this.num = number;
			this.crawlerthread = thread;
			this.crawlerthread.messageEmit += new EventHandler(OnMessage);
			this.Build();
			//Add button buttonClear
			this.buttonClear = new Button();
			this.buttonClear.Image = new Image(System.IO.Path.Combine(System.AppDomain.CurrentDomain.BaseDirectory, 
			                                                          "Images" + 
			                                                          System.IO.Path.DirectorySeparatorChar + 
			                                                          "clear.png"));
			this.buttonClear.ImagePosition = PositionType.Left;
			this.buttonClear.Label = Catalog.GetString("Clear");
			this.AddActionWidget(buttonClear, 0);
			this.buttonClear.Clicked += new EventHandler(OnButtonClearClicked);
			this.buttonClear.ShowAll();
			//Add button buttonOk 
			this.buttonOk = new Button();
			this.buttonOk.Image = new Image(System.IO.Path.Combine(System.AppDomain.CurrentDomain.BaseDirectory, 
			                                                       "Images" + 
			                                                       System.IO.Path.DirectorySeparatorChar + 
			                                                       "ok.png"));
			this.buttonOk.ImagePosition = PositionType.Left;
			this.buttonOk.Label = Catalog.GetString("_OK");
			this.buttonOk.UseUnderline = true;
			this.AddActionWidget(buttonOk, -5);
			this.buttonOk.Clicked += new EventHandler(OnButtonOkClicked);
			this.buttonOk.CanDefault = true;
			this.buttonOk.GrabFocus();
			this.buttonOk.ShowAll();
			this.Icon = new Gdk.Pixbuf(System.IO.Path.Combine(System.AppDomain.CurrentDomain.BaseDirectory, 
			                                                  "Images" + 
			                                                  System.IO.Path.DirectorySeparatorChar + "grubby.ico"));
			this.txtdetails.Buffer.InsertText += new InsertTextHandler(this.OnInsertText);
			this.crawlerstatus = new string[11]{Catalog.GetString("Stopped"),
				Catalog.GetString("Paused"),
				Catalog.GetString("In Queue"),
				Catalog.GetString("Downloading"),
				Catalog.GetString("Download Error"),
				Catalog.GetString("Crawling"),
				Catalog.GetString("Uploading"),
				Catalog.GetString("Upload Error"),
				Catalog.GetString("Waiting"),
				Catalog.GetString("Invalid arc file"),
				Catalog.GetString("Program error")
			};
			this.crawlermessage = new string[24]{Catalog.GetString("Waiting in queue for download new workunit"),
				Catalog.GetString("Waiting in queue for upload workunit"),
				Catalog.GetString("Invalid login. Check your setting for username and password."),
				Catalog.GetString("Request timeout."),
				Catalog.GetString("Problems with connection to Internet."),
				Catalog.GetString("Unknown error"),
				Catalog.GetString("Done"),
				Catalog.GetString("File checked and compressed, ready to upload."),
				Catalog.GetString("Workunit uploaded"),
				Catalog.GetString("Invalid workunit, you must try crawl it again"),
				Catalog.GetString("Timeout reached when uploading. Please check your internet connection"),
				Catalog.GetString("Cannot connect to server. Try upload workunit later."),
				Catalog.GetString("Internal Server Error. Try download workunit later."),
				Catalog.GetString("No server response. Try upload workunit again."),
				Catalog.GetString("Program error appear and crawler was stopped."),
				Catalog.GetString("Internal Server Error. Try upload .arc file later."),
				Catalog.GetString("Sitemap uploaded."),
				Catalog.GetString("Internal Server Error. Sitemap cannot be uploaded."),
				Catalog.GetString("Generate sitemap."),
				Catalog.GetString("Upload .arc files to this server is temporary disabled. Please try later."),
				Catalog.GetString("You was sent this file earlier. You cannot send this same .arc file twice."),
				Catalog.GetString("Someone was sent this same URL's earlier."),
				Catalog.GetString("No workunits available at this moment. Please try later."),
				Catalog.GetString("Upload sitemaps to this server is disabled.")
			};
			this.status = -1;
			TextTag link = new TextTag("link");
			link.Foreground = "#0000FF";
			link.Underline = Pango.Underline.Single;
			this.txtdetails.Buffer.TagTable.Add(link);
			this.crawlerthread.OnMessageEmit(EventArgs.Empty);
		}
		
		/// <summary>
		/// Function destroy dialog. Raised when buttonOk clicked.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> unused
		/// </param>
		/// <param name="e">
		/// A <see cref="System.EventArgs"/> unused
		/// </param>
		void OnButtonOkClicked (object sender, System.EventArgs e)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (e != null)
			{
				e = null;
			}
			this.crawlerthread.messageEmit -= new EventHandler(this.OnMessage);
			this.Destroy();
		}
		
		/// <summary>
		/// Function clear console. Raised when buttonClear clicked.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> unused
		/// </param>
		/// <param name="e">
		/// A <see cref="System.EventArgs"/> unused
		/// </param>
		void OnButtonClearClicked (object sender, System.EventArgs e)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (e != null)
			{
				e = null;
			}
			this.txtdetails.Buffer.Clear();
		}
		
		/// <summary>
		/// Move cursor in console to end. Raised when text was inserted in txtdetails.
		/// </summary>
		/// <param name="o">
		/// A <see cref="System.Object"/> unused
		/// </param>
		/// <param name="args">
		/// A <see cref="InsertTextArgs"/> unused
		/// </param>
		void OnInsertText (object o, InsertTextArgs args)
		{
			if (o != null)
			{
				o = null;
			}
			if (args != null)
			{
				args = null;
			}
			if (this.txtdetails.Buffer.InsertMark.Deleted)
			{
				TextMark insert = this.txtdetails.Buffer.CreateMark("insert", this.txtdetails.Buffer.EndIter, 
				                                                    false);
				this.txtdetails.Buffer.GetIterAtMark(insert);
			}
			this.txtdetails.ScrollMarkOnscreen(this.txtdetails.Buffer.InsertMark);
		}
		
		/// <summary>
		/// Function return TextIter for TextView buffer.
		/// </summary>
		/// <returns>
		/// A <see cref="TextIter"/> on which text should be inserted.
		/// </returns>
		TextIter GetIter()
		{
			TextIter iter;
			try
			{
				iter = this.txtdetails.Buffer.EndIter;
			}
			catch (NullReferenceException)
			{
				iter = this.txtdetails.Buffer.GetIterAtLine(0);
			}
			return iter;
		}
		
		/// <summary>
		/// Function update GUI. Raised when crawler thread emits message event.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> unused
		/// </param>
		/// <param name="e">
		/// A <see cref="EventArgs"/> unused
		/// </param>
		void OnMessage(object sender, EventArgs e)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (e != null)
			{
				e = null;
			}
			string tdate = DateTime.Now.ToString("HH:mm:ss");
			int crawlstatus = this.crawlerthread.GetField(this.num, "status");
			int message = this.crawlerthread.GetField(this.num, "message");
			string servermessage = this.crawlerthread.GetStringField(this.num, "servermessage");
			if (this.status != crawlstatus)
			{
				Application.Invoke(delegate {
					TextIter titer = this.GetIter();
					this.txtdetails.Buffer.Insert(ref titer, tdate + " " + 
					                              this.crawlerstatus[crawlstatus] + 
					                              Environment.NewLine);
				});
				this.status = crawlstatus;
			}
			if (message != -1)
			{
				Application.Invoke(delegate {
					TextIter titer = this.GetIter();
					this.txtdetails.Buffer.Insert(ref titer, tdate + " " + 
					                              this.crawlermessage[message] + 
					                              Environment.NewLine);
					this.crawlerthread.SetField(this.num, "message", 0);
				});
			}
			if (servermessage.Length > 0)
			{
				Application.Invoke(delegate {
					TextIter titer = this.GetIter();
					this.txtdetails.Buffer.Insert(ref titer, tdate + " " + Catalog.GetString("Server response:") + " " +
					                              servermessage + Environment.NewLine);
					this.crawlerthread.SetField(this.num, "servermessage", 0);
				});
			}
			if (this.status == 5)
			{
				string cururl = this.crawlerthread.GetStringField(this.num, "cururl");
				int curstatus = this.crawlerthread.GetField(this.num, "curstatus");
				if (cururl.Length > 0)
				{
					Application.Invoke(delegate {
						TextIter titer = this.GetIter();
						this.txtdetails.Buffer.Insert(ref titer, tdate + " [" + this.crawlerthread.GetField(this.num, "curiter").ToString() + 
						                              "/" + (this.crawlerthread.GetField(this.num, "iter") - 1).ToString() + "] ");
						this.txtdetails.Buffer.InsertWithTagsByName(ref titer, cururl, "link");
						this.crawlerthread.SetField(this.num, "cururl", 0);
					});
				}
				if (curstatus != 0)
				{
					Application.Invoke(delegate {
						TextIter titer = this.GetIter();
						this.txtdetails.Buffer.Insert(ref titer, " " + Catalog.GetString("[Status:") + " " + 
						                              curstatus.ToString() + "]" + Environment.NewLine);
					});
					this.crawlerthread.SetField(this.num, "curstatus", 0);
				}
			}
		}

		/// <summary>
		/// Function open browser with selected from txtview link. Raised when mouse click on txtview.
		/// </summary>
		/// <param name="o">
		/// A <see cref="System.Object"/> unused.
		/// </param>
		/// <param name="args">
		/// A <see cref="Gtk.ButtonReleaseEventArgs"/> give information about which button is pressed.
		/// </param>
		[GLib.ConnectBefore]
		void OnTxtdetailsButtonReleaseEvent (object o, Gtk.ButtonReleaseEventArgs args)
		{
			if (o != null)
			{
				o = null;
			}
			if (args.Event.Button == 1)
			{
				TextIter titer = this.txtdetails.Buffer.GetIterAtMark(this.txtdetails.Buffer.InsertMark);
				for (int i = 0; i < titer.Tags.Length; i++)
				{
					if (titer.Tags[i].Name == "link")
					{
						TextIter startline = titer;
						bool result = this.txtdetails.BackwardDisplayLine(ref startline);
						TextIter endline = titer;
						result = this.txtdetails.ForwardDisplayLineEnd(ref endline);
						if (result)
						{
						}
						string line = this.txtdetails.Buffer.GetText(startline, endline, false);
						string[] splitline = line.Split(' ');
						System.Diagnostics.Process proc = System.Diagnostics.Process.Start(splitline[2]);
						proc.Close();
						splitline = null;
						line = null;
						break;
					}
				}
			}
		}

		/// <summary>
		/// Function change cursor. Raised when mouse cursor moved.
		/// </summary>
		/// <param name="o">
		/// A <see cref="System.Object"/> unused
		/// </param>
		/// <param name="args">
		/// A <see cref="Gtk.MotionNotifyEventArgs"/> coordinates of mouse cursor.
		/// </param>
		void OnTxtdetailsMotionNotifyEvent (object o, Gtk.MotionNotifyEventArgs args)
		{
			if (o != null)
			{
				o = null;
			}
			int x = 0;
			int y = 0;
			this.txtdetails.WindowToBufferCoords(TextWindowType.Text, (int)args.Event.X, (int)args.Event.Y, out x, out y);
			TextIter titer = this.txtdetails.GetIterAtLocation(x, y);
			bool cursor = false;
			for (int i = 0; i < titer.Tags.Length; i++)
			{
				if (titer.Tags[i].Name == "link")
				{
					this.txtdetails.GetWindow(TextWindowType.Text).Cursor = new Gdk.Cursor(Gdk.CursorType.Hand1);
					cursor = true;
					break;
				}
			}
			if (!cursor)
			{
				this.txtdetails.GetWindow(TextWindowType.Text).Cursor = null;
			}
		}
		
		/// <summary>
		/// Function override Gtk.Object.Dispose function.
		/// </summary>
		public override void Dispose()
		{
			this.GtkScrolledWindow.Dispose();
			this.txtdetails.Dispose();
			this.buttonClear.Dispose();
			this.buttonOk.Dispose();
			base.Dispose();
		}
	}
}
