// ErrorDialog.cs
// 
// Copyright (C) 2008,2009,2010 Bartek Jasicki
//
// This file is part of Grubng.
//
// Grubng is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

/**@file ErrorDialog.cs
 * Provide GTK Dialog for show info about unhandled exception
 * @fn virtual void Grubng::ErrorDialog::Build()
 * Create part of GUI and show it to user*/

using System;
using Gtk;

namespace Grubng
{
    
	/// <summary>
	/// Provide GTK Dialog for show info about unhandled exception.
	/// </summary>
    internal sealed partial class ErrorDialog : Gtk.Dialog
    {
		/// <summary>
		/// GTK Button to close dialog.
		/// </summary>
		Button buttonCancel;
        
		/// <summary>
		/// Standard class constructor. Create dialog and show it to user.
		/// </summary>
		/// <param name="errorinfo">
		/// A <see cref="System.String"/> Additional informations about error.
		/// </param>
        public ErrorDialog(string errorinfo)
        {
            this.Build();
			//Add button buttonCancel 
			this.buttonCancel = new Button();
			this.buttonCancel.Image = new Image(System.IO.Path.Combine(System.AppDomain.CurrentDomain.BaseDirectory, "Images" + System.IO.Path.DirectorySeparatorChar + "ok.png"));
			this.buttonCancel.ImagePosition = PositionType.Left;
			this.buttonCancel.Label = Mono.Unix.Catalog.GetString("_OK");
			this.buttonCancel.UseUnderline = true;
			this.AddActionWidget(buttonCancel, -6);
			this.buttonCancel.Clicked += new EventHandler(OnButtonCancelClicked);
			this.buttonCancel.CanDefault = true;
			this.buttonCancel.GrabFocus();
			this.buttonCancel.ShowAll();
			this.Icon = new Gdk.Pixbuf(System.IO.Path.Combine(System.AppDomain.CurrentDomain.BaseDirectory, "Images" + System.IO.Path.DirectorySeparatorChar + "grubby.ico"));
			TextIter iter = this.txterror.Buffer.EndIter;
			this.txterror.Buffer.Insert(ref iter, errorinfo);
        }

		/// <summary>
		/// Destroy error dialog.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> unused.
		/// </param>
		/// <param name="e">
		/// A <see cref="System.EventArgs"/> unused.
		/// </param>
        void OnButtonCancelClicked (object sender, System.EventArgs e)
        {
			if (sender != null)
			{
				sender = null;
			}
			if (e != null)
			{
				e = null;
			}
            this.Destroy();
        }
		
		/// <summary>
		/// Function override Gtk.Object.Dispose function.
		/// </summary>
		public override void Dispose()
		{
			this.vbox3.Dispose();
			this.label2.Dispose();
			this.label3.Dispose();
			this.label4.Dispose();
			this.expinfo.Dispose();
			this.GtkScrolledWindow.Dispose();
			this.txterror.Dispose();
			this.GtkLabel.Dispose();
			this.buttonCancel.Dispose();
			base.Dispose();
		}
    }
}
