// HelpDialog.cs
// 
// Copyright (C) 2008,2009,2010 Bartek Jasicki
//
// This file is part of Grubng.
//
// Grubng is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

/**@file HelpDialog.cs
 * Provide GTK Dialog for show text file
 * @fn virtual void Grubng::HelpDialog::Build()
 * Create part of GUI and show it to user*/

using System;
using System.IO;
using Gtk;

namespace Grubng
{
	
	/// <summary>
	/// Provide GTK Dialog for show text file.
	/// </summary>
	internal sealed partial class HelpDialog : Gtk.Dialog
	{
		/// <summary>
		/// GTK Button to close dialog.
		/// </summary>
		Button buttonOk;
		
		/// <summary>
		/// Class contructor, create dialog and show it to user.
		/// </summary>
		/// <param name="filename">
		/// A <see cref="System.String"/> Full path (including file name) of file text to show
		/// </param>
		public HelpDialog(string filename)
		{
			this.Build();
			//Add button buttonOk 
			this.buttonOk = new Button();
			this.buttonOk.Image = new Image(System.IO.Path.Combine(System.AppDomain.CurrentDomain.BaseDirectory, "Images" + System.IO.Path.DirectorySeparatorChar + "ok.png"));
			this.buttonOk.ImagePosition = PositionType.Left;
			this.buttonOk.Label = Mono.Unix.Catalog.GetString("_OK");
			this.buttonOk.UseUnderline = true;
			this.AddActionWidget(buttonOk, -5);
			this.buttonOk.Clicked += new EventHandler(OnButtonOkClicked);
			this.buttonOk.CanDefault = true;
			this.buttonOk.GrabFocus();
			this.buttonOk.ShowAll();
			this.Title = filename;
			this.Icon = new Gdk.Pixbuf(System.IO.Path.Combine(System.AppDomain.CurrentDomain.BaseDirectory, "Images" + System.IO.Path.DirectorySeparatorChar + "grubby.ico"));
			//Read README.txt
			FileStream file = System.IO.File.OpenRead(System.IO.Path.Combine(System.AppDomain.CurrentDomain.BaseDirectory, filename));
			StreamReader stream = new StreamReader(file);
			txthelp.Buffer.Text = stream.ReadToEnd();
			stream.Close();
			file.Close();
		}

		/// <summary>
		/// Destroy help dialog.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> unused.
		/// </param>
		/// <param name="e">
		/// A <see cref="System.EventArgs"/> unused.
		/// </param>
		void OnButtonOkClicked (object sender, System.EventArgs e)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (e != null)
			{
				e = null;
			}
			this.Destroy();
		}
		
		/// <summary>
		/// Function override Gtk.Object.Dispose function.
		/// </summary>
		public override void Dispose()
		{
			this.GtkScrolledWindow.Dispose();
			this.txthelp.Dispose();
			this.buttonOk.Dispose();
			base.Dispose();
		}
	}
}
