// LogonDialog.cs
// 
// Copyright (C) 2008,2009,2010 Bartek Jasicki
//
// This file is part of Grubng.
//
//  Grubng is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//   Grubng is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with Grubng.  If not, see <http://www.gnu.org/licenses/>.
//

/**@file LogonDialog.cs
 * Provide GTK Dialog for set username and password
 * @fn virtual void Grubng::LogonDialog::Build()
 * Create part of GUI and show it to user*/

using System;
using System.IO;
using Gtk;
using Grubng;
using Mono.Unix;

namespace Grubng
{
	
	/// <summary>
	/// Provide GTK Dialog for set username and password.
	/// </summary>
	internal sealed partial class LogonDialog : Gtk.Dialog
	{
		/// <summary>
		/// GTK Button for exit from program.
		/// </summary>
		Button buttonCancel;
		/// <summary>
		/// GTK Button for create grub.xml file.
		/// </summary>
		Button buttonOk;
		
		/// <summary>
		/// Standard class constructor. Create dialog and show it to user.
		/// </summary>
		public LogonDialog()
		{
			this.Build();
			//Add button buttonCancel
			this.buttonCancel = new Button();
			this.buttonCancel.Image = new Image(System.IO.Path.Combine(System.AppDomain.CurrentDomain.BaseDirectory, "Images" + System.IO.Path.DirectorySeparatorChar + "cancel.png"));
			this.buttonCancel.ImagePosition = PositionType.Left;
			this.buttonCancel.Label = Catalog.GetString("_Cancel");
			this.buttonCancel.UseUnderline = true;
			this.AddActionWidget(buttonCancel, -6);
			this.buttonCancel.ShowAll();
			//Add button buttonOk 
			this.buttonOk = new Button();
			this.buttonOk.Image = new Image(System.IO.Path.Combine(System.AppDomain.CurrentDomain.BaseDirectory, "Images" + System.IO.Path.DirectorySeparatorChar + "ok.png"));
			this.buttonOk.ImagePosition = PositionType.Left;
			this.buttonOk.Label = Catalog.GetString("_OK");
			this.buttonOk.UseUnderline = true;
			this.AddActionWidget(buttonOk, -5);
			this.buttonOk.Clicked += new EventHandler(OnButtonOkClicked);
			this.buttonOk.ShowAll();
			//Add dialog icon
			this.Icon = new Gdk.Pixbuf(System.IO.Path.Combine(System.AppDomain.CurrentDomain.BaseDirectory, "Images" + System.IO.Path.DirectorySeparatorChar + "grubby.ico"));
		}

		/// <summary>
		/// Function check for empty fields in form. If all is ok, create
		/// new grub.xml file. If not - show info about it.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> unused.
		/// </param>
		/// <param name="e">
		/// A <see cref="System.EventArgs"/> unused.
		/// </param>
		void OnButtonOkClicked (object sender, System.EventArgs e)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (e != null)
			{
				e = null;
			}
			//Check for empty fields
			bool checkuser = String.IsNullOrEmpty(edituser.Text);
			bool checkpass = String.IsNullOrEmpty(editpass.Text);
			if (checkuser || checkpass)
			{
				MessageDialog mesempty = new MessageDialog (this, 
				                                            DialogFlags.DestroyWithParent,
				                                            MessageType.Error, 
				                                            ButtonsType.Close, Catalog.GetString("Fill empty fields"));
				int result = mesempty.Run();
				if (result < 0)
				{
					mesempty.Destroy();
					mesempty.Dispose();
				}
			}
			else
			{
				Config conf = new Config();
				conf.CreateConfig(edituser.Text, editpass.Text);
			}
		}

		/// <summary>
		/// Function open web browser on wikia create account page. Raised when btncreate clicked.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> unused.
		/// </param>
		/// <param name="e">
		/// A <see cref="System.EventArgs"/> unused.
		/// </param>
		void OnBtncreateClicked (object sender, System.EventArgs e)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (e != null)
			{
				e = null;
			}
			System.Diagnostics.Process proc = System.Diagnostics.Process.Start("http://grub.org/?q=en/user/register");
			proc.Close();
		}
		
		/// <summary>
		/// Function override Gtk.Object.Dispose function.
		/// </summary>
		public override void Dispose()
		{
			this.table2.Dispose();
			this.editpass.Dispose();
			this.edituser.Dispose();
			this.label2.Dispose();
			this.label3.Dispose();
			this.btncreate.Dispose();
			this.buttonCancel.Dispose();
			this.buttonOk.Dispose();
			base.Dispose();
		}
	}
}
