// MainWindowGUI.cs
// 
// Copyright (C) 2008,2009,2010,2011 Bartek Jasicki
//
// This file is part of Grubng.
//
// Grubng is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

/**
 * @file MainWindowGUI.cs
 * Manually generated GTK# GUI*/

using System;
using Gtk;
using Grubng;
using Mono.Unix;

namespace Grubng
{
	
	/// <summary>
	/// Provide GTK Window for program and core functions.
	/// </summary>
	internal sealed partial class MainWindow : Gtk.Window
	{
		/// <summary>
		/// GTK Button for show console (details)
		/// </summary>
		Button btndetails;
		/// <summary>
		/// GTK Button for show stats
		/// </summary>
		Button btnstats;
		/// <summary>
		/// GTK Button to start/pause crawling
		/// </summary>
		Button btnrun;
		/// <summary>
		/// GTK Button to exit from program
		/// </summary>
		Button btnquit;
		/// <summary>
		/// GTK Label for show program uptime
		/// </summary>
		Label lbluptime;
		/// <summary>
		/// GTK Button for show options
		/// </summary>
		Button btnoptions;
		
		/// <summary>
		/// Function manually createe few elements of GUI
		/// </summary>
		void BuildGUI()
		{
			TreeViewColumn column;
			column = this.treedet.AppendColumn("#", new CellRendererText(), "text", 0);
			column.Resizable = true;
			column.SortOrder = SortType.Ascending;
			column.SortColumnId = 0;
			column.Clicked += new EventHandler(OnColumnClick);
			column = this.treedet.AppendColumn(Catalog.GetString("Status"), new CellRendererText(), "text", 1);
			column.Resizable = true;
			column.SortOrder = SortType.Ascending;
			column.SortColumnId = 1;
			column.Clicked += new EventHandler(OnColumnClick);
			column = this.treedet.AppendColumn(Catalog.GetString("Progress"), new CellRendererProgress(), "value", 2);
			column.Resizable = true;
			column.SortOrder = SortType.Ascending;
			column.SortColumnId = 2;
			column.Clicked += new EventHandler(OnColumnClick);
			column = this.treedet.AppendColumn(Catalog.GetString("Name"), new CellRendererText(), "text", 3);
			column.Resizable = true;
			column.SortOrder = SortType.Ascending;
			column.SortColumnId = 3;
			column.Clicked += new EventHandler(OnColumnClick);
			column = this.treedet.AppendColumn(Catalog.GetString("Downloaded"), new CellRendererText(), "text", 4);
			column.Resizable = true;
			column.SortOrder = SortType.Ascending;
			column.SortColumnId = 4;
			column.Clicked += new EventHandler(OnColumnClick);
			CellRendererText txt = new CellRendererText();
			column = this.treedet.AppendColumn(Catalog.GetString("MB Used"), txt, "text", 5);
			column.Resizable = true;
			column.SortOrder = SortType.Ascending;
			column.SortColumnId = 5;
			column.SetCellDataFunc(txt, new CellLayoutDataFunc(FormatDiskUsage));
			column.Clicked += new EventHandler(OnColumnClick);
			this.treedet.HeadersClickable = true;
			this.treedet.RowActivated += OnTreedetRowActivated;
			this.treedet.ButtonReleaseEvent += OnTreedetButtonReleaseEvent;
			this.btnmoreoptions.Clicked += OnPreferencesActivated;
			this.imgstatus.Pixbuf = this.stopimg;
			this.cbtnauto.Toggled += OnAutomaticToggled;
			this.hsmaxcrawlers.ValueChanged += OnHsmaxcrawlersValueChanged;
			this.hsmaxdownload.ValueChanged += OnHsmaxdownloadValueChanged;
			this.hsmaxupload.ValueChanged += OnHsmaxuploadValueChanged;
			this.sbtnmaxcrawlers.ValueChanged += OnSbtnmaxcrawlersValueChanged;
			this.sbtnmaxdownload.ValueChanged += OnSbtnmaxdownloadValueChanged;
			this.sbtnmaxupload.ValueChanged += OnSbtnmaxuploadValueChanged;
			//Add button btndetails
			this.btndetails = new Button();
			this.btndetails.Image = new Image(System.IO.Path.Combine(System.AppDomain.CurrentDomain.BaseDirectory, 
			                                                         "Images" + System.IO.Path.DirectorySeparatorChar + "icon_crawler.png"));
			this.btndetails.ImagePosition = PositionType.Left;
			this.btndetails.Label = Catalog.GetString("Details");
			this.btndetails.Clicked += new EventHandler(this.OnBtndetailsClicked);
			this.vbuttonbox3.Add(this.btndetails);
			//Add button btnstats
			this.btnstats = new Button();
			this.btnstats.Image = new Image(System.IO.Path.Combine(System.AppDomain.CurrentDomain.BaseDirectory, 
			                                                       "Images" + System.IO.Path.DirectorySeparatorChar + "icon_control.png"));
			this.btnstats.ImagePosition = PositionType.Left;
			this.btnstats.Label = Catalog.GetString("Stats");
			this.btnstats.Sensitive = false;
			this.btnstats.Clicked += new EventHandler(OnBtnstatsClicked);
			this.vbuttonbox3.Add(this.btnstats);
			//Add button btnoptions
			this.btnoptions = new Button();
			this.btnoptions.Image = new Image(System.IO.Path.Combine(System.AppDomain.CurrentDomain.BaseDirectory, 
			                                                       "Images" + System.IO.Path.DirectorySeparatorChar + "grub-preferences.png"));
			this.btnoptions.ImagePosition = PositionType.Left;
			this.btnoptions.Label = Catalog.GetString("Options");
			this.btnoptions.Clicked += new EventHandler(OnBtnoptionsClicked);
			this.vbuttonbox3.Add(this.btnoptions);
			this.vbuttonbox3.ShowAll();
			//Add button btnquit
			this.btnquit = new Button();
			this.btnquit.Image = new Image(System.IO.Path.Combine(System.AppDomain.CurrentDomain.BaseDirectory, 
			                                                      "Images" + System.IO.Path.DirectorySeparatorChar + "quit.png"));
			this.btnquit.ImagePosition = PositionType.Left;
			this.btnquit.Label = Catalog.GetString("_Quit");
			this.btnquit.UseUnderline = true;
			this.btnquit.Clicked += new EventHandler(OnQuitActivated);
			this.hbuttonbox1.Add(this.btnquit);
			//Add button btnrun
			this.btnrun = new Button();
			this.btnrun.Image = new Image(System.IO.Path.Combine(System.AppDomain.CurrentDomain.BaseDirectory, "Images" + System.IO.Path.DirectorySeparatorChar + "run.png"));
			this.btnrun.ImagePosition = PositionType.Left;
			this.btnrun.Label = Catalog.GetString("Start");
			this.btnrun.Clicked += new EventHandler(OnBtnrunClicked);
			this.hbuttonbox1.Add(this.btnrun);
			this.hbuttonbox1.ShowAll();
			this.btnrun.CanDefault = true;
			this.btnrun.GrabFocus();
			//Add label lbluptime
			this.lbluptime = new Label(Catalog.GetString("00:00:00"));
			this.table4.Add(this.lbluptime);
			Table.TableChild tabchild = ((Table.TableChild)(this.table4[this.lbluptime]));
			tabchild.LeftAttach = (uint)2;
			tabchild.RightAttach = (uint)3;
			tabchild.XOptions = (AttachOptions)4;
			tabchild.YOptions = (AttachOptions)4;
			lbluptime.Show();
			//Add images
			this.image9.Pixbuf = new Gdk.Pixbuf(System.IO.Path.Combine(System.AppDomain.CurrentDomain.BaseDirectory, "Images" + System.IO.Path.DirectorySeparatorChar + "time.png"));
			this.image8.Pixbuf = new Gdk.Pixbuf(System.IO.Path.Combine(System.AppDomain.CurrentDomain.BaseDirectory, "Images" + System.IO.Path.DirectorySeparatorChar + "curtime.png"));
			this.image103.Pixbuf = new Gdk.Pixbuf(System.IO.Path.Combine(System.AppDomain.CurrentDomain.BaseDirectory, "Images" + System.IO.Path.DirectorySeparatorChar + "worksend.png"));
			this.image104.Pixbuf = new Gdk.Pixbuf(System.IO.Path.Combine(System.AppDomain.CurrentDomain.BaseDirectory, "Images" + System.IO.Path.DirectorySeparatorChar + "worksend.png"));
			this.image53.Pixbuf = new Gdk.Pixbuf(System.IO.Path.Combine(System.AppDomain.CurrentDomain.BaseDirectory, "Images" + System.IO.Path.DirectorySeparatorChar + "links.png"));
			this.image56.Pixbuf = new Gdk.Pixbuf(System.IO.Path.Combine(System.AppDomain.CurrentDomain.BaseDirectory, "Images" + System.IO.Path.DirectorySeparatorChar + "links.png"));
			this.image490.Pixbuf = new Gdk.Pixbuf(System.IO.Path.Combine(System.AppDomain.CurrentDomain.BaseDirectory, "Images" + System.IO.Path.DirectorySeparatorChar + "yes.png"));
			this.image491.Pixbuf = new Gdk.Pixbuf(System.IO.Path.Combine(System.AppDomain.CurrentDomain.BaseDirectory, "Images" + System.IO.Path.DirectorySeparatorChar + "yes.png"));
			this.image13.Pixbuf = new Gdk.Pixbuf(System.IO.Path.Combine(System.AppDomain.CurrentDomain.BaseDirectory, "Images" + System.IO.Path.DirectorySeparatorChar + "no.png"));
			this.image15.Pixbuf = new Gdk.Pixbuf(System.IO.Path.Combine(System.AppDomain.CurrentDomain.BaseDirectory, "Images" + System.IO.Path.DirectorySeparatorChar + "no.png"));
			this.image12.Pixbuf = new Gdk.Pixbuf(System.IO.Path.Combine(System.AppDomain.CurrentDomain.BaseDirectory, "Images" + System.IO.Path.DirectorySeparatorChar + "errors.png"));
			this.image14.Pixbuf = new Gdk.Pixbuf(System.IO.Path.Combine(System.AppDomain.CurrentDomain.BaseDirectory, "Images" + System.IO.Path.DirectorySeparatorChar + "errors.png"));
			this.image232.Pixbuf = new Gdk.Pixbuf(System.IO.Path.Combine(System.AppDomain.CurrentDomain.BaseDirectory, "Images" + System.IO.Path.DirectorySeparatorChar + "received.png"));
			this.image233.Pixbuf = new Gdk.Pixbuf(System.IO.Path.Combine(System.AppDomain.CurrentDomain.BaseDirectory, "Images" + System.IO.Path.DirectorySeparatorChar + "received.png"));
			this.image10.Pixbuf = new Gdk.Pixbuf(System.IO.Path.Combine(System.AppDomain.CurrentDomain.BaseDirectory, "Images" + System.IO.Path.DirectorySeparatorChar + "send.png"));
			this.image11.Pixbuf = new Gdk.Pixbuf(System.IO.Path.Combine(System.AppDomain.CurrentDomain.BaseDirectory, "Images" + System.IO.Path.DirectorySeparatorChar + "send.png"));
			this.image1.Pixbuf = new Gdk.Pixbuf(System.IO.Path.Combine(System.AppDomain.CurrentDomain.BaseDirectory, "Images" + System.IO.Path.DirectorySeparatorChar + "diskusage.png"));
			this.image2.Pixbuf = new Gdk.Pixbuf(System.IO.Path.Combine(System.AppDomain.CurrentDomain.BaseDirectory, "Images" + System.IO.Path.DirectorySeparatorChar + "invalidarc.png"));
			this.image3.Pixbuf = new Gdk.Pixbuf(System.IO.Path.Combine(System.AppDomain.CurrentDomain.BaseDirectory, "Images" + System.IO.Path.DirectorySeparatorChar + "invalidarc.png"));
			//Set window icon
			this.Icon = new Gdk.Pixbuf(System.IO.Path.Combine(System.AppDomain.CurrentDomain.BaseDirectory, "Images" + System.IO.Path.DirectorySeparatorChar + "grubby.ico"));
			this.DeleteEvent += OnDeleteEvent;
		}
	
		/// <summary>
		/// Function update sbtnmaxcrawlers value. Raised when value hsmaxcrawlers changed
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> Unused
		/// </param>
		/// <param name="e">
		/// A <see cref="EventArgs"/> Unused
		/// </param>
		void OnHsmaxcrawlersValueChanged(object sender, EventArgs e)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (e != null)
			{
				e = null;
			}
			if (this.sbtnmaxcrawlers.Value != this.hsmaxcrawlers.Value)
			{
				this.sbtnmaxcrawlers.Value = this.hsmaxcrawlers.Value;
			}
		}
		
		/// <summary>
		/// Function update sbtnmaxdownload value. Raised when value hsmaxdownload changed
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> Unused
		/// </param>
		/// <param name="e">
		/// A <see cref="EventArgs"/> Unused
		/// </param>
		void OnHsmaxdownloadValueChanged(object sender, EventArgs e)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (e != null)
			{
				e = null;
			}
			if (this.sbtnmaxdownload.Value != this.hsmaxdownload.Value)
			{
				this.sbtnmaxdownload.Value = this.hsmaxdownload.Value;
			}
		}
		
		/// <summary>
		/// Function update sbtnmaxupload value. Raised when value hsmaxcupload changed
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> Unused
		/// </param>
		/// <param name="e">
		/// A <see cref="EventArgs"/> Unused
		/// </param>
		void OnHsmaxuploadValueChanged(object sender, EventArgs e)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (e != null)
			{
				e = null;
			}
			if (this.sbtnmaxupload.Value != this.hsmaxupload.Value)
			{
				this.sbtnmaxupload.Value = this.hsmaxupload.Value;
			}
		}
		
		/// <summary>
		/// Function update hsmaxcrawlers value and settings. Raised when sbtnmaxcrawlers value changed.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> Unused
		/// </param>
		/// <param name="e"> Unused
		/// A <see cref="System.EventArgs"/>
		/// </param>
		void OnSbtnmaxcrawlersValueChanged (object sender, System.EventArgs e)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (e != null)
			{
				e = null;
			}
			if (this.hsmaxcrawlers.Value != this.sbtnmaxcrawlers.Value)
			{
				this.hsmaxcrawlers.Value = this.sbtnmaxcrawlers.Value;
			}
		}
		
		/// <summary>
		/// Function update hsmaxdownload value and settings. Raised when sbtnmaxdownload value changed.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> Unused
		/// </param>
		/// <param name="e"> Unused
		/// A <see cref="System.EventArgs"/>
		/// </param>
		void OnSbtnmaxdownloadValueChanged (object sender, System.EventArgs e)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (e != null)
			{
				e = null;
			}
			if (this.hsmaxdownload.Value != this.sbtnmaxdownload.Value)
			{
				this.hsmaxdownload.Value = this.sbtnmaxdownload.Value;
			}
		}
		
		/// <summary>
		/// Function update hsmaxupload value and settings. Raised when sbtnmaxupload value changed.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> Unused
		/// </param>
		/// <param name="e"> Unused
		/// A <see cref="System.EventArgs"/>
		/// </param>
		void OnSbtnmaxuploadValueChanged (object sender, System.EventArgs e)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (e != null)
			{
				e = null;
			}
			if (this.hsmaxupload.Value != this.sbtnmaxupload.Value)
			{
				this.hsmaxupload.Value = this.sbtnmaxupload.Value;
			}
		}
		
		/// <summary>
		/// Function format disk usage column values (one digit after decimal point).
		/// </summary>
		/// <param name="layout">
		/// A <see cref="CellLayout"/> Unused.
		/// </param>
		/// <param name="cell">
		/// A <see cref="CellRenderer"/> cell in which value been changed.
		/// </param>
		/// <param name="model">
		/// A <see cref="TreeModel"/> all values in table.
		/// </param>
		/// <param name="iter">
		/// A <see cref="TreeIter"/> current iter in table.
		/// </param>
		void FormatDiskUsage(CellLayout layout, CellRenderer cell, TreeModel model, TreeIter iter)
		{
			double diskusage = Math.Round(Convert.ToDouble(model.GetValue(iter, 5)), 1);
			CellRendererText txt = (CellRendererText)cell;
			txt.Text = diskusage.ToString();
		}
	}
}
