// PassDialog.cs
// 
// Copyright (C) 2008,2010 Bartek Jasicki
//
// This file is part of Grubng.
//
//  Grubng is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//   Grubng is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with Grubng.  If not, see <http://www.gnu.org/licenses/>.
//

/**@file PassDialog.cs
 * Provide GTK Dialog for root password for wide system update
 * @fn virtual void Grubng::PassDialog::Build()
 * Create part of GUI and show it to user*/

using System;
using Gtk;
using Mono.Unix;

namespace Grubng
{
	
	/// <summary>
	/// Provide GTK Dialog for root password for wide system update.
	/// </summary>
	internal sealed partial class PassDialog : Gtk.Dialog
	{
		/// <summary>
		/// GTK Button for close dialog.
		/// </summary>
		Button buttonOk;
		/// <summary>
		/// Root password.
		/// </summary>
		string rootpass;
		
		/// <summary>
		/// tandard class constructor. Create dialog, and show it to user.
		/// </summary>
		public PassDialog()
		{
			this.Build();
			//Add button buttonOk 
			this.buttonOk = new Button();
			this.buttonOk.Image = new Image(System.IO.Path.Combine(System.AppDomain.CurrentDomain.BaseDirectory, "Images" + System.IO.Path.DirectorySeparatorChar + "ok.png"));
			this.buttonOk.ImagePosition = PositionType.Left;
			this.buttonOk.Label = Catalog.GetString("_OK");
			this.buttonOk.UseUnderline = true;
			this.AddActionWidget(buttonOk, -5);
			this.buttonOk.Clicked += new EventHandler(OnButtonOkClicked);
			this.buttonOk.ShowAll();
			//Add dialog icon
			this.Icon = new Gdk.Pixbuf(System.IO.Path.Combine(System.AppDomain.CurrentDomain.BaseDirectory, "Images" + System.IO.Path.DirectorySeparatorChar + "grubby.ico"));
		}
		
		/// <summary>
		/// Function set root password.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> unused.
		/// </param>
		/// <param name="e">
		/// A <see cref="System.EventArgs"/> unused.
		/// </param>
		void OnButtonOkClicked (object sender, System.EventArgs e)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (e != null)
			{
				e = null;
			}
			this.rootpass = this.edtpass.Text;
		}
		
		/// <summary>
		/// Property set private field rootpass.
		/// </summary>
		public string RootPass
		{
			get {return this.rootpass;}
			set {this.rootpass = value;}
		}
		
		/// <summary>
		/// Function override Gtk.Object.Dispose function.
		/// </summary>
		public override void Dispose()
		{
			this.hbox1.Dispose();
			this.label2.Dispose();
			this.edtpass.Dispose();
			this.lblinfo.Dispose();
			this.buttonOk.Dispose();
			base.Dispose();
		}
	}
}
