// PrefDialog.cs
// 
// Copyright (C) 2008,2009,2010,2011 Bartek Jasicki
//
// This file is part of Grubng.
//
// Grubng is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

/**@file PrefDialog.cs
 * Provide GTK Dialog for set program preferences.
 * @fn virtual void Grubng::PrefDialog::Build()
 * Create part of GUI and show it to user.*/

using System;
using Mono.Unix;
using Gtk;

namespace Grubng
{
	
	/// <summary>
	/// Provide GTK Dialog for set program preferences.
	/// </summary>
	internal sealed partial class PrefDialog : Gtk.Dialog
	{
		/// <summary>
		/// Object from Config class. Provide function to save, read and write setting to grub.xml file.
		/// </summary>
		Config conf;
		/// <summary>
		/// Used when user change setting for work directory. If false - nothing happens.
		/// If true - buttons Ok and Apply are available.
		/// </summary>
		bool newdir;
		/// <summary>
		/// Used to check availability few GUI elements. If true, user can set program dir. 
		/// Can be true only when program not working
		/// </summary>
		bool btnavailable;
		/// <summary>
		/// Is true if stats are cleared.
		/// </summary>
		bool cleared;
		/// <summary>
		/// GTK Button for clear stats in grub.xml file
		/// </summary>
		Button btnclearstats;
		/// <summary>
		/// GTK Button for close dialog without saving
		/// </summary>
		Button btncancel;
		/// <summary>
		/// GTK Button for close dialog and save changes to grub.xml file.
		/// </summary>
		Button btnok;
		/// <summary>
		/// GTK Button for save changes to grub.xml file.
		/// </summary>
		Button btnapply;
		/// <summary>
		/// GTK Button for check for updates.
		/// </summary>
		Button btncheck;
		
		/// <summary>
		/// Standard class constructor. Create GUI and show it to user, show actual program settings.
		/// </summary>
		public PrefDialog()
		{
			this.Build();
			//Add button btnclearstats
			this.btnclearstats = new Button();
			this.btnclearstats.Image = new Image(System.IO.Path.Combine(System.AppDomain.CurrentDomain.BaseDirectory, "Images" + 
			                                                            System.IO.Path.DirectorySeparatorChar + "clear.png"));
			this.btnclearstats.ImagePosition = PositionType.Left;
			this.btnclearstats.Label = Catalog.GetString("Clear stats");
			this.btnclearstats.Clicked += new EventHandler(this.OnBtnclearstatsClicked);
			this.table2.Add(this.btnclearstats);
			Table.TableChild tabchild = ((Table.TableChild)(this.table2[this.btnclearstats]));
			tabchild.TopAttach = (uint)4;
			tabchild.BottomAttach = (uint)5;
			tabchild.XOptions = (AttachOptions)0;
			tabchild.YOptions = (AttachOptions)4;
			this.btnclearstats.ShowAll();
			//Add button btncancel
			this.btncancel = new Button();
			this.btncancel.Image = new Image(System.IO.Path.Combine(System.AppDomain.CurrentDomain.BaseDirectory, "Images" + 
			                                                        System.IO.Path.DirectorySeparatorChar + "cancel.png"));
			this.btncancel.ImagePosition = PositionType.Left;
			this.btncancel.Label = Catalog.GetString("_Cancel");
			this.btncancel.UseUnderline = true;
			this.AddActionWidget(btncancel, -6);
			this.btncancel.Clicked += new EventHandler(OnBtncancelClicked);
			this.btncancel.CanDefault = true;
			this.btncancel.GrabFocus();
			this.btncancel.ShowAll();
			//Add button btnapply 
			this.btnapply = new Button();
			this.btnapply.Image = new Image(System.IO.Path.Combine(System.AppDomain.CurrentDomain.BaseDirectory, "Images" + 
			                                                       System.IO.Path.DirectorySeparatorChar + "apply.png"));
			this.btnapply.ImagePosition = PositionType.Left;
			this.btnapply.Label = Catalog.GetString("Apply");
			this.AddActionWidget(btnapply, 0);
			this.btnapply.Name = "btnapply";
			this.btnapply.Clicked += new EventHandler(OnBtnokClicked);
			this.btnapply.ShowAll();
			//Add button btnok 
			this.btnok = new Button();
			this.btnok.Image = new Image(System.IO.Path.Combine(System.AppDomain.CurrentDomain.BaseDirectory, "Images" + 
			                                                    System.IO.Path.DirectorySeparatorChar + "ok.png"));
			this.btnok.ImagePosition = PositionType.Left;
			this.btnok.Label = Catalog.GetString("_OK");
			this.btnok.UseUnderline = true;
			this.AddActionWidget(btnok, -5);
			this.btnok.Name = "btnok";
			this.btnok.Clicked += new EventHandler(OnBtnokClicked);
			this.btnok.ShowAll();
			//Add button btncheck
			this.btncheck = new Button();
			this.btncheck.Image = new Image(System.IO.Path.Combine(System.AppDomain.CurrentDomain.BaseDirectory, "Images" + 
			                                                    System.IO.Path.DirectorySeparatorChar + "update.png"));
			this.btncheck.ImagePosition = PositionType.Left;
			this.btncheck.Label = Catalog.GetString("Check Now");
			this.btncheck.Clicked += new EventHandler(this.OnBtncheckClicked);
			this.hbox1.Add(this.btncheck);
			Box.BoxChild boxchild = ((Box.BoxChild)(this.hbox1[this.btncheck]));
            boxchild.Position = 0;
            boxchild.Expand = false;
            boxchild.Fill = false;
			this.btncheck.ShowAll();
			//Add dialog icon
			this.Icon = new Gdk.Pixbuf(System.IO.Path.Combine(System.AppDomain.CurrentDomain.BaseDirectory, "Images" + 
			                                                  System.IO.Path.DirectorySeparatorChar + "grubby.ico"));
			this.conf = new Config();
			string[] nodes = new string[] {
				"/configuration/login/username",
				"/configuration/login/password",
				"/configuration/work/workserver",
				"/configuration/work/workdir",
				"/configuration/work/compression",
				"/configuration/work/automatic",
				"/configuration/other/runhidden",
				"/configuration/work/proxystatus",
				"/configuration/other/updateserver",
				"/configuration/other/checkupdates",
				"/configuration/other/unstable",
				"/configuration/work/createsitemaps",
				"/configuration/other/sitemapserver",
				"/configuration/other/deletesitemap",
				"/configuration/other/enableremote",
				"/configuration/other/remoteport"
			};
			string[] values = Config.ReadFew(nodes, this.conf.FilePath);
			this.edtuser.Text = values[0];
			this.edtpass.Text = values[1];
			this.edtworkserv.Text = values[2];
			bool result = this.btndir.SetCurrentFolder(values[3]);
			if (!result)
			{
				result = this.btndir.SetCurrentFolder(Environment.GetFolderPath(Environment.SpecialFolder.Personal));
			}
			if (values[4] == ".gz")
			{
				this.rbtngzip.Active = true;
			}
			else
			{
				this.rbtnlzma.Active = true;
			}
			if (values[5] == "Y")
			{
				this.cbtnauto.Active = true;
			}
			else
			{
				this.cbtnauto.Active = false;
			}
			if (values[6] == "Y")
			{
				this.cbtnrunhidden.Active = true;
			}
			else
			{
				this.cbtnrunhidden.Active = false;
			}
			switch (values[7])
			{
			case "none":
				this.rbtnnone.Active = true;
				break;
			case "default":
				this.rbtndefault.Active = true;
				break;
			case "custom":
				this.rbtncustom.Active = true;
				nodes = new string[] {
					"/configuration/work/proxyaddress",
					"/configuration/work/proxyport",
					"/configuration/work/proxylogin",
					"/configuration/work/proxypass"};
				string[] proxyvals = Config.ReadFew(nodes, this.conf.FilePath);
				this.editproxy.Text = proxyvals[0];
				this.editport.Text = proxyvals[1];
				this.editproxylogin.Text = proxyvals[2];
				this.editproxypass.Text = proxyvals[3];
				proxyvals = null;
				break;
			}
			nodes = null;
			this.editupdater.Text = values[8];
			if (values[9] == "Y")
			{
				this.cbtnupdater.Active = true;
				this.rbtnunstable.Sensitive = true;
				this.rbtnunstable.Active = true;
				this.rbtnstable.Sensitive = true;
				this.btncheck.Sensitive = true;
			}
			else
			{
				this.cbtnupdater.Active = false;
				this.rbtnunstable.Sensitive = false;
				this.btncheck.Sensitive = false;
			}
			if (values[10] == "Y")
			{
				this.rbtnunstable.Active = true;
			}
			else
			{
				this.rbtnstable.Active = true;
			}
			//Set sitemaps settings
			if (values[11] == "Y")
			{
				this.cbtnenablesitemap.Active = true;
			}
			else
			{
				this.cbtnenablesitemap.Active = false;
			}
			this.edtmapserver.Text = values[12];
			if (values[13] == "Y")
			{
				this.cbtnsitemap.Active = true;
			}
			else
			{
				this.cbtnsitemap.Active = false;
			}
			if (values[14] == "Y")
			{
				this.cbtnenableremote.Active = true;
			}
			else
			{
				this.cbtnenableremote.Active = false;
			}
			this.edtremoteport.Text = values[15];
			values = null;
			this.btnok.Sensitive = false;
			this.btnapply.Sensitive = false;
		}
		
		/// <summary>
		/// Function destroy preferences dialog.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> Unused
		/// </param>
		/// <param name="e">
		/// A <see cref="System.EventArgs"/> Unused
		/// </param>
		void OnBtncancelClicked (object sender, System.EventArgs e)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (e != null)
			{
				e = null;
			}
			this.Destroy();
		}

		/// <summary>
		/// Set available buttons Ok and Apply. Buttons are available when any settings
		/// are changed by user.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> Unused
		/// </param>
		/// <param name="e">
		/// A <see cref="System.EventArgs"/> Unused
		/// </param>
		void OnEdtuserChanged (object sender, System.EventArgs e)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (e != null)
			{
				e = null;
			}
			if (!this.btnok.Sensitive)
			{
				this.btnok.Sensitive = true;
				this.btnapply.Sensitive = true;
			}
		}

		/// <summary>
		/// Save changes in setting to configuration file and close Preferences dialog. Raised when button Ok or Apply is clicked.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> GTK Button which emit event
		/// </param>
		/// <param name="e">
		/// A <see cref="System.EventArgs"/> unused
		/// </param>
		void OnBtnokClicked (object sender, System.EventArgs e)
		{
			if (e != null)
			{
				e = null;
			}
			if (SavePrefs())
			{
				Button clicked = (Button)sender;
				if (clicked.Name == "btnok")
				{
					this.Destroy();
				}
				else
				{
					this.btnapply.Sensitive = false;
				}
			}
			else
			{
				this.Respond(1);
			}
		}

		/// <summary>
		/// Check for changed working directory setting. If user change working directory setting,
		/// this function unblock buttons Ok and Apply.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> Unused
		/// </param>
		/// <param name="e">
		/// A <see cref="System.EventArgs"/> Unused
		/// </param>
		void OnBtndirCurrentFolderChanged (object sender, System.EventArgs e)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (e != null)
			{
				e = null;
			}
			if (this.newdir)
			{
				if (!this.btnok.Sensitive)
				{
					this.btnok.Sensitive = true;
					this.btnapply.Sensitive = true;
				}
			}
			this.newdir = true;
		}

		/// <summary>
		/// Function clear statistics in grub.xml file.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> unused
		/// </param>
		/// <param name="e">
		/// A <see cref="System.EventArgs"/> unused
		/// </param>
		void OnBtnclearstatsClicked (object sender, System.EventArgs e)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (e != null)
			{
				e = null;
			}
			this.cleared = true;
			if (this.cleared)
			{
				string[] xmlvalues = new string[] {"0", "0", "0", "0", "0", "0", "0", "0"};
				string[] xmlnodes = new string[] {
					"/configuration/statistics/worksend",
					"/configuration/statistics/urlcrawled",
					"/configuration/statistics/success",
					"/configuration/statistics/failure",
					"/configuration/statistics/errors",
					"/configuration/statistics/mbreceived",
					"/configuration/statistics/mbsend",
					"/configuration/statistics/invalidarc"
				};
				Config.WriteFew(xmlvalues, xmlnodes, this.conf.FilePath);
			}
		}

		/// <summary>
		/// Function set availability of few settings.
		/// </summary>
		/// <param name="o">
		/// A <see cref="System.Object"/>
		/// </param>
		/// <param name="args">
		/// A <see cref="Gtk.SwitchPageArgs"/>
		/// </param>
		void OnNotebook1SwitchPage (object o, Gtk.SwitchPageArgs args)
		{
			if (o != null)
			{
				o = null;
			}
			if (args != null)
			{
				args = null;
			}
			this.btndir.Sensitive = this.btnavailable;
			if (this.cbtnupdater.Active)
			{
				this.btncheck.Sensitive = this.btnavailable;
			}
		}
		
		/// <summary>
		/// Function check fields in forms and if they are correct, save settings to grub.xml file.
		/// </summary>
		/// <returns>
		/// A <see cref="System.Boolean"/> True if configuration file was updated, otherwise
		/// false.
		/// </returns>
		bool SavePrefs()
		{
			//Check for empty fields
			bool checkuser = String.IsNullOrEmpty(this.edtuser.Text);
			bool checkpass = String.IsNullOrEmpty(this.edtpass.Text);
			bool checkwork = String.IsNullOrEmpty(this.edtworkserv.Text);
			if (checkuser || checkpass || checkwork)
			{
				this.ShowMessage(Catalog.GetString("Fill empty fields"), MessageType.Error, ButtonsType.Close);
				return false;
			}
			//Check for valid URL of server with workunits
			if (!this.ValidURL(this.edtworkserv.Text))
			{
				return false;
			}
			//Check sitemap server option only when creating sitemap is enabled
			if (this.cbtnenablesitemap.Active)
			{
				if (this.edtmapserver.Text.Length == 0)
				{
					this.ShowMessage(Catalog.GetString("Fill sitemap server settings"), MessageType.Error, ButtonsType.Close);
					return false;
				}
				if (!this.ValidURL(this.edtmapserver.Text))
				{
					return false;
				}
			}
			//Check updater server option only when auto-updater is enabled
			if (this.cbtnupdater.Active)
			{
				if (this.editupdater.Text.Length == 0)
				{
					this.ShowMessage(Catalog.GetString("Fill updater settings"), MessageType.Error, ButtonsType.Close);
					return false;
				}
				if (!this.ValidURL(this.editupdater.Text))
				{
					return false;
				}
			}
			string[] xmlnodes = new string[] 
			{
				"/configuration/work/proxystatus",
				"/configuration/work/proxyaddress",
				"/configuration/work/proxyport",
				"/configuration/work/proxylogin",
				"configuration/work/proxypass",
				"/configuration/login/username",
				"/configuration/login/password",
				"/configuration/work/workdir",
				"/configuration/work/automatic",
				"/configuration/other/runhidden",
				"/configuration/work/workserver",
				"/configuration/work/compression",
				"/configuration/other/updateserver",
				"/configuration/other/checkupdates",
				"/configuration/other/unstable",
				"/configuration/work/createsitemaps",
				"/configuration/other/sitemapserver",
				"/configuration/other/deletesitemap",
				"/configuration/other/enableremote",
				"/configuration/other/remoteport"
			};
			string[] xmlvalues = new string[xmlnodes.Length];
			//If set custom proxy setting, check for empty fields
			if (this.rbtncustom.Active)
			{
				bool checkproxy = String.IsNullOrEmpty(this.editproxy.Text);
				bool checkport = String.IsNullOrEmpty(this.editport.Text);
				if (checkproxy || checkport)
				{
					this.ShowMessage(Catalog.GetString("Fill proxy settings"), MessageType.Error, ButtonsType.Close);
					return false;
				}
				//Check valid proxy url
				checkproxy = this.ValidURL(this.editproxy.Text);
				if (!checkproxy)
				{
					return false;
				}
				//Check valid proxy port
				System.Text.RegularExpressions.Regex rx = new System.Text.RegularExpressions.Regex(@"^[1-9][0-9]*$");
				if (!rx.IsMatch(this.editport.Text))
				{
					this.ShowMessage(Catalog.GetString("Value must be a number."), MessageType.Error, ButtonsType.Close);
					return false;
				}
				xmlvalues[0] = "custom";
				xmlvalues[1] = this.editproxy.Text;
				xmlvalues[2] = this.editport.Text;
				xmlvalues[3] = this.editproxylogin.Text;
				xmlvalues[4] = this.editproxypass.Text;
			}
			else
			{
				xmlvalues[1] = String.Empty;
				xmlvalues[2] = String.Empty;
				xmlvalues[3] = String.Empty;
				xmlvalues[4] = String.Empty;
				if (this.rbtndefault.Active)
				{
					xmlvalues[0] = "default";
				}
				else
				{
					xmlvalues[0] = "none";
				}
			}
			xmlvalues[5] = this.edtuser.Text;
			xmlvalues[6] = this.edtpass.Text;
			xmlvalues[7] = this.btndir.CurrentFolder + System.IO.Path.DirectorySeparatorChar;
			if (this.cbtnauto.Active)
			{
				xmlvalues[8] = "Y";
			}
			else
			{
				xmlvalues[8] = "N";
			}
			if (this.cbtnrunhidden.Active)
			{
				xmlvalues[9] = "Y";
			}
			else
			{
				xmlvalues[9] = "N";
			}
			xmlvalues[10] = this.edtworkserv.Text;
			if (this.rbtngzip.Active)
			{
				xmlvalues[11] = ".gz";
			}
			else
			{
				xmlvalues[11] = ".lzma";
			}
			xmlvalues[12] = this.editupdater.Text;
			if (this.cbtnupdater.Active)
			{
				xmlvalues[13] = "Y";
			}
			else
			{
				xmlvalues[13] = "N";
			}
			if (this.rbtnunstable.Active)
			{
				xmlvalues[14] = "Y";
			}
			else
			{
				xmlvalues[14] = "N";
			}
			if (this.cbtnenablesitemap.Active)
			{
				xmlvalues[15] = "Y";
			}
			else
			{
				xmlvalues[15] = "N";
			}
			xmlvalues[16] = this.edtmapserver.Text;
			if (this.cbtnsitemap.Active)
			{
				xmlvalues[17] = "Y";
			}
			else
			{
				xmlvalues[17] = "N";
			}
			if (this.cbtnenableremote.Active)
			{
				//Check valid remote control port
				System.Text.RegularExpressions.Regex rx2 = new System.Text.RegularExpressions.Regex(@"^[1-9][0-9]*$");
				if (!rx2.IsMatch(this.edtremoteport.Text))
				{
					this.ShowMessage(Catalog.GetString("Value must be a number."), MessageType.Error, ButtonsType.Close);
					return false;
				}
				xmlvalues[18] = "Y";
			}
			else
			{
				xmlvalues[18] = "N";
			}
			xmlvalues[19] = this.edtremoteport.Text;
			Config.WriteFew(xmlvalues, xmlnodes, this.conf.FilePath);
			return true;
		}

		/// <summary>
		/// Function enable or disable fields to configure proxy server. Raised when rbtncustom toggled.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> unused.
		/// </param>
		/// <param name="e">
		/// A <see cref="System.EventArgs"/> unused.
		/// </param>
		void OnRbtncustomToggled (object sender, System.EventArgs e)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (e != null)
			{
				e = null;
			}
			this.lblproxy.Sensitive = !this.lblproxy.Sensitive;
			this.lblport.Sensitive = !this.lblport.Sensitive;
			this.editproxy.Sensitive = !this.editproxy.Sensitive;
			this.editport.Sensitive = !this.editport.Sensitive;
			this.lblproxylogin.Sensitive = !this.lblproxylogin.Sensitive;
			this.lblproxypass.Sensitive = !this.lblproxypass.Sensitive;
			this.editproxylogin.Sensitive = !this.editproxylogin.Sensitive;
			this.editproxypass.Sensitive = !this.editproxypass.Sensitive;
		}

		/// <summary>
		/// Function enable or disable autoupdater. Raised when cbtnupdater clicked.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> unused.
		/// </param>
		/// <param name="e">
		/// A <see cref="System.EventArgs"/> unused.
		/// </param>
		void OnCbtnupdaterClicked (object sender, System.EventArgs e)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (e != null)
			{
				e = null;
			}
			if (this.cbtnupdater.Active)
			{
				this.rbtnunstable.Sensitive = true;
				this.rbtnstable.Sensitive = true;
				this.btncheck.Sensitive = this.btnavailable;
			}
			else
			{
				this.rbtnunstable.Sensitive = false;
				this.rbtnstable.Sensitive = false;
				this.btncheck.Sensitive = false;
			}
		}
		
		/// <summary>
		/// Function enable or disable remote control. Raised when cbtnenableremote clicked.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> unused.
		/// </param>
		/// <param name="e">
		/// A <see cref="System.EventArgs"/> unused.
		/// </param>
		void OnCbtnenableremoteToggled (object sender, System.EventArgs e)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (e != null)
			{
				e = null;
			}
			this.lblremoteport.Sensitive = !this.lblremoteport.Sensitive;
			this.edtremoteport.Sensitive = !this.edtremoteport.Sensitive;
		}
		
		//// <value>
		/// Property set private field btnavailable.
		/// </value>
		public bool ButtonAvailable
		{
			get {return this.btnavailable;}
			set {this.btnavailable = value;}
		}
		
		//// <value>
		/// Property set private fields cleared.
		/// </value>
		public bool Cleared
		{
			get {return this.cleared;}
			set {this.cleared = value;}
		}
		
		/// <summary>
		/// Function check valid url.
		/// </summary>
		/// <param name="url">
		/// A <see cref="System.String"/> URL to check.
		/// </param>
		/// <returns>
		/// A <see cref="System.Boolean"/> true if URL is valid, otherwise return false.
		/// </returns>
		bool ValidURL(string url)
		{
			System.Text.RegularExpressions.Regex rx = new System.Text.RegularExpressions.Regex(@"^(ht|f)tp(s?)\:\/\/[0-9a-zA-Z]([-.\w]*[0-9a-zA-Z])*(:(0-9)*)*(\/?)([a-zA-Z0-9\-\.\?\,\'\/\\\+&amp;%\$#_~]*)?$");
			if (rx.IsMatch(url))
			{
				return true;
			}
			else
			{
				this.ShowMessage(Catalog.GetString("Invalid url"), MessageType.Error, ButtonsType.Close);
				return false;
			}
		}
		
		/// <summary>
		/// Function override Gtk.Object.Dispose() function.
		/// </summary>
		public override void Dispose()
		{
			this.notebook1.Dispose();
			this.table1.Dispose();
			this.edtpass.Dispose();
			this.edtuser.Dispose();
			this.label3.Dispose();
			this.label4.Dispose();
			this.lblproxylogin.Dispose();
			this.btndir.Dispose();
			this.lblproxypass.Dispose();
			this.vbox3.Dispose();
			this.table3.Dispose();
			this.edtworkserv.Dispose();
			this.label9.Dispose();
			this.label7.Dispose();
			this.vbox4.Dispose();
			this.rbtnnone.Dispose();
			this.rbtndefault.Dispose();
			this.rbtncustom.Dispose();
			this.table4.Dispose();
			this.editport.Dispose();
			this.editproxy.Dispose();
			this.lblport.Dispose();
			this.lblproxy.Dispose();
			this.label11.Dispose();
			this.table2.Dispose();
			this.cbtnauto.Dispose();
			this.label6.Dispose();
			this.vbox5.Dispose();
			this.hbox2.Dispose();
			this.hbox1.Dispose();
			this.label14.Dispose();
			this.editupdater.Dispose();
			this.cbtnupdater.Dispose();
			this.rbtnstable.Dispose();
			this.rbtnunstable.Dispose();
			this.btncheck.Dispose();
			this.label13.Dispose();
			this.vbox6.Dispose();
			this.hbox3.Dispose();
			this.label18.Dispose();
			this.edtmapserver.Dispose();
			this.cbtnsitemap.Dispose();
			this.label17.Dispose();
			this.btnclearstats.Dispose();
			this.btncancel.Dispose();
			this.btnapply.Dispose();
			this.btnok.Dispose();
			this.label1.Dispose();
			this.editproxylogin.Dispose();
			this.editproxypass.Dispose();
			this.cbtnrunhidden.Dispose();
			this.label8.Dispose();
			this.rbtngzip.Dispose();
			this.rbtnlzma.Dispose();
			this.cbtnenablesitemap.Dispose();
			this.cbtnenableremote.Dispose();
			this.hbox4.Dispose();
			this.lblremoteport.Dispose();
			this.edtremoteport.Dispose();
			base.Dispose();
		}

		/// <summary>
		/// Function check for updates of client. Raised when btncheck clicked.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> unused.
		/// </param>
		/// <param name="e">
		/// A <see cref="System.EventArgs"/> unused.
		/// </param>
		void OnBtncheckClicked (object sender, System.EventArgs e)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (e != null)
			{
				e = null;
			}
			this.btncheck.Sensitive = false;
			this.GdkWindow.Cursor = new Gdk.Cursor(Gdk.CursorType.Watch);
			while (Application.EventsPending())
			{
				Application.RunIteration();
			}
			string rootpass = String.Empty;
			string unstable = Config.ReadConfig("/configuration/other/unstable", conf.FilePath);
			System.Reflection.Assembly asm = System.Reflection.Assembly.GetExecutingAssembly();
			string version = asm.GetName().Version.Major.ToString() + "." + asm.GetName().Version.Minor.ToString() + "." + asm.GetName().Version.Build.ToString();
			Updater upd = new Updater(version);
			string update = upd.CheckUpdate(unstable);
			if ((update == "current") || (update == "older"))
			{
				bool rootneed = false;
				string[] path = System.AppDomain.CurrentDomain.BaseDirectory.Split('/');
				if (path[1] != "home")
				{
					rootneed = true;
				}
				if (rootneed)
				{
					PassDialog pass = new PassDialog();
					pass.Run();
					rootpass = pass.RootPass;
					while (String.IsNullOrEmpty(rootpass))
					{
						pass.Run();
						rootpass = pass.RootPass;
					}
					pass.Destroy();
					pass.Dispose();
				}
				string[] updated = Updater.Update(update, unstable, rootpass);
				this.GdkWindow.Cursor = new Gdk.Cursor(Gdk.CursorType.LeftPtr);
				while (Application.EventsPending())
				{
					Application.RunIteration();
				}
				MessageDialog mesupd = new MessageDialog (null, 
				                                          DialogFlags.DestroyWithParent,
				                                          MessageType.Info, 
				                                          ButtonsType.Ok, 
				                                          Catalog.GetString("Client successfully updated. ") + 
				                                          Catalog.GetString("You must restart it again. "));
				mesupd.Text += Environment.NewLine + Environment.NewLine + Catalog.GetString("Updated files:") + Environment.NewLine;
				for (int i = 0; i < updated.Length; i++)
				{
					mesupd.Text += (i + 1).ToString() + " " + updated[i] + Environment.NewLine;
				}
				mesupd.Run();
				mesupd.Destroy();
				mesupd.Dispose();
			}
			else
			{
				this.GdkWindow.Cursor = new Gdk.Cursor(Gdk.CursorType.LeftPtr);
				while (Application.EventsPending())
				{
					Application.RunIteration();
				}
				string message;
				if (update == "none")
				{
					message = Catalog.GetString("No available updates.");
				}
				else
				{
					message = Catalog.GetString("Update failed.");
				}
				MessageDialog mesupd2 = new MessageDialog (null, 
				                                           DialogFlags.DestroyWithParent,
				                                           MessageType.Info, 
				                                           ButtonsType.Ok, 
				                                           message);
				mesupd2.Run();
				mesupd2.Destroy();
				mesupd2.Dispose();
			}
			this.btncheck.Sensitive = true;
		}
		
		/// <summary>
		/// Function show message dialog with information to user
		/// </summary>
		/// <param name="message">
		/// A <see cref="System.String"/> translated information to show to user
		/// </param>
		/// <param name="mtype">
		/// A <see cref="Gtk.MessageType"/> message type
		/// </param>
		/// <param name="btype">
		/// A <see cref="Gtk.ButtonsType"/> buttons on dialog
		/// </param>
		void ShowMessage(string message, MessageType mtype, ButtonsType btype)
		{
			MessageDialog mesdialog = new MessageDialog (this, 
			                                             DialogFlags.DestroyWithParent,
			                                             mtype, 
			                                             btype, 
			                                             message);
			int result = mesdialog.Run();
			result++;
			mesdialog.Destroy();
			mesdialog.Dispose();
		}
	}
}
