// ShowSitemapDialog.cs
// 
// Copyright (C) 2008,2009,2010 Bartek Jasicki
//
// This file is part of Grubng.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

/**@file ShowSitemapDialog.cs
 * Provide GTK Dialog for edit and send sitemap.
 * @fn virtual void Grubng::ShowSitemapDialog::Build()
 * Create part of GUI and show it to user*/

using Gtk;
using System;
using System.Xml;
using System.IO;
using Mono.Unix;

namespace Grubng
{
	
	/**Provide GTK Dialog for edit and send sitemap.*/
	public partial class ShowSitemapDialog : Gtk.Dialog
	{
		
		/**GTK Button for exit from dialog.*/
		private Button buttonCancel;
		/**GTK Button for send created sitemap.*/
		private Button buttonSend;
		/**GTK Button for save created sitemap.*/
		private Button buttonSave;
		/**GTK Button for add new link to sitemap.*/
		private Button btnadd;
		/**GTK Button for delete link from sitemap.*/
		private Button btndelete;
		/// <summary>
		/// Full path to sitemap with filename.
		/// </summary>
		string mapfile;
		/// <summary>
		/// GTK TreeStore to store urls from sitemap.
		/// </summary>
		TreeStore store;
		/// <summary>
		/// List of TreeIters for urls from sitemap.
		/// </summary>
		System.Collections.Generic.List<TreeIter> iter;
		
		/// <summary>
		/// Standard class constructor. Create dialog and show it to user.
		/// </summary>
		/// <param name="sitemap">
		/// A <see cref="System.String"/> Full path (including filename) to sitemap.
		/// </param>
		public ShowSitemapDialog(string sitemap)
		{
			this.mapfile = sitemap;
			this.Build();
			CellRendererText renderer = new CellRendererText();
			renderer.Editable = true;
			renderer.Edited += new EditedHandler(OnCellEdit);
			TreeViewColumn column = this.treemap.AppendColumn(Catalog.GetString("Link"), renderer, "text", 0);
			column.Resizable = true;
			column.SortOrder = SortType.Ascending;
			column.SortColumnId = 0;
			this.store = new TreeStore(typeof (string));
			this.iter = new System.Collections.Generic.List<TreeIter>();
			//Add button buttonCancel
			this.buttonCancel = new Button();
			this.buttonCancel.Image = new Image(System.IO.Path.Combine(System.AppDomain.CurrentDomain.BaseDirectory, 
			                                                           "Images" + System.IO.Path.DirectorySeparatorChar + 
			                                                           "cancel.png"));
			this.buttonCancel.ImagePosition = PositionType.Left;
			this.buttonCancel.Label = Catalog.GetString("_Cancel");
			this.buttonCancel.UseUnderline = true;
			this.AddActionWidget(buttonCancel, -6);
			this.buttonCancel.ShowAll();
			//Add button buttonSave
			this.buttonSave = new Button();
			this.buttonSave.Image = new Image(System.IO.Path.Combine(System.AppDomain.CurrentDomain.BaseDirectory, 
			                                                       "Images" + System.IO.Path.DirectorySeparatorChar + 
			                                                       "save.png"));
			this.buttonSave.ImagePosition = PositionType.Left;
			this.buttonSave.Label = Catalog.GetString("S_ave");
			this.buttonSave.UseUnderline = true;
			this.AddActionWidget(buttonSave, 2);
			this.buttonSave.Clicked += new EventHandler(OnButtonSaveClicked);
			this.buttonSave.Sensitive = false;
			this.buttonSave.ShowAll();
			//Add button buttonSend 
			this.buttonSend = new Button();
			this.buttonSend.Image = new Image(System.IO.Path.Combine(System.AppDomain.CurrentDomain.BaseDirectory, 
			                                                       "Images" + System.IO.Path.DirectorySeparatorChar + 
			                                                       "ok.png"));
			this.buttonSend.ImagePosition = PositionType.Left;
			this.buttonSend.Label = Catalog.GetString("_Send");
			this.buttonSend.UseUnderline = true;
			this.buttonSend.Sensitive = false;
			this.AddActionWidget(buttonSend, 1);
			this.buttonSend.Clicked += new EventHandler(OnButtonSendClicked);
			this.buttonSend.ShowAll();
			this.buttonSend.CanDefault = true;
			//Add button btnadd
			this.btnadd = new Button();
			this.btnadd.Image = new Image(System.IO.Path.Combine(System.AppDomain.CurrentDomain.BaseDirectory, 
			                                                     "Images" + System.IO.Path.DirectorySeparatorChar + 
			                                                     "add.png"));
			this.btnadd.ImagePosition = PositionType.Left;
			this.btnadd.Label = Catalog.GetString("Add link");
			this.btnadd.Clicked += new EventHandler(this.OnBtnaddClicked);
			this.vbuttonbox1.Add(this.btnadd);
			//Add button btndelete
			this.btndelete = new Button();
			this.btndelete.Image = new Image(System.IO.Path.Combine(System.AppDomain.CurrentDomain.BaseDirectory, 
			                                                        "Images" + System.IO.Path.DirectorySeparatorChar + 
			                                                        "delete.png"));
			this.btndelete.ImagePosition = PositionType.Left;
			this.btndelete.Label = Catalog.GetString("Delete link");
			this.btndelete.Sensitive = false;
			this.btndelete.Clicked += new EventHandler(OnBtndeleteClicked);
			this.vbuttonbox1.Add(this.btndelete);
			this.vbuttonbox1.ShowAll();
			//Add dialog icon
			this.Icon = new Gdk.Pixbuf(System.IO.Path.Combine(System.AppDomain.CurrentDomain.BaseDirectory, 
			                                                  "Images" + System.IO.Path.DirectorySeparatorChar + 
			                                                  "grubby.ico"));
			//Read sitemap file (if exist)
			if (System.IO.File.Exists(this.mapfile))
			{
				this.buttonSend.Sensitive = this.ReadSitemap();
			}
			this.treemap.Model = this.store;
			this.buttonSend.GrabFocus();
		}
		
		/// <summary>
		/// Function read existing sitemap and show it in treemap.
		/// </summary>
		/// <returns>
		/// A <see cref="System.Boolean"/> true if sitemap was not empty, otherwise false.
		/// </returns>
		bool ReadSitemap()
		{
			string[] extension = this.mapfile.Split('.');
			int index = extension.Length - 1;
			if (extension[index] == "gz")
			{
				FileStream file = File.OpenRead(this.mapfile);
				this.mapfile = this.mapfile.Substring(0, (this.mapfile.Length - 3));
				byte[] buffer = new byte[1024];
				FileStream unfile = File.Create(this.mapfile);
				System.IO.Compression.GZipStream gzstream = new System.IO.Compression.GZipStream(file,
				                                                                                 System.IO.Compression.CompressionMode.Decompress);
				int bytesRead = 0;
				while (true)
				{
					bytesRead = gzstream.Read(buffer, 0, buffer.Length);
					if (bytesRead == 0)
					{
						break;
					}
					unfile.Write(buffer, 0, bytesRead);
				}
				gzstream.Close();
				gzstream.Dispose();;
				buffer = null;
				unfile.Close();
				unfile.Dispose();
				file.Close();
				file.Dispose();
				File.Delete(this.mapfile + ".gz");
			}
			XmlDocument doc = new XmlDocument();
			doc.Load(this.mapfile);
			XmlNode node = doc.SelectSingleNode("/");
			XmlElement url = (XmlElement)node.ChildNodes[1];
			if (url.GetElementsByTagName("url").Count == 0)
			{
				doc = null;
				return false;
			}
			for (int i = 0; i < url.GetElementsByTagName("url").Count; i++)
			{
				this.iter.Add(this.store.AppendValues(url.GetElementsByTagName("url")[i].FirstChild.InnerText));
			}
			doc = null;
			return true;
		}
		
		/**Send created sitemap on server.
		 * Raised when buttonsend clicked.
		 * @param sender Unused
		 * @param e Unused*/
		private void OnButtonSendClicked (object sender, System.EventArgs e)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (e != null)
			{
				e = null;
			}
			SitemapDialog sitemapdlg = new SitemapDialog(this.mapfile);
			int result = sitemapdlg.Run();
			if (result < 0)
			{
				sitemapdlg.Destroy();
				sitemapdlg.Dispose();
				this.buttonCancel.Click();
			}
		}
		
		/**Save list of urls from TreeModel to sitemap.
		 * Raised when buttonsave clicked.
		 * @param sender Unused
		 * @param e Unused*/
		private void OnButtonSaveClicked (object sender, System.EventArgs e)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (e != null)
			{
				e = null;
			}
			string tmpmap = this.mapfile.Substring(0, (this.mapfile.Length - 3)) + "tmp";
			FileStream file = File.Create(tmpmap);
			StreamWriter mapwriter = new StreamWriter(file);
			foreach (TreeIter iter2 in this.iter)
			{
				if (this.store.IterIsValid(iter2))
				{
					mapwriter.WriteLine(this.store.GetValue(iter2, 0));
				}
			}
			mapwriter.Dispose();
			file.Dispose();
			File.Delete(this.mapfile);
			Sitemap sitemap = new Sitemap(this.mapfile);
			sitemap.CreateSitemap(false);
			sitemap = null;
			this.buttonSave.Sensitive = false;
			this.buttonSend.Sensitive = true;
		}
		
		/**Function add new empty link to sitemap.
		 * Raised when btnadd clicked
		 * @param sender Unused
		 * @param e Unused*/
		private void OnBtnaddClicked (object sender, System.EventArgs e)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (e != null)
			{
				e = null;
			}
			this.iter.Add(this.store.AppendValues("http://"));
			if (!this.buttonSave.Sensitive)
			{
				this.buttonSave.Sensitive = true;
				this.buttonSend.Sensitive = false;
			}
		}
		
			              
		/// <summary>
		/// Function delete link from sitemap. Raised when btndelete was clicked.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> unused.
		/// </param>
		/// <param name="e">
		/// A <see cref="System.EventArgs"/> unused.
		/// </param>
		void OnBtndeleteClicked (object sender, System.EventArgs e)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (e != null)
			{
				e = null;
			}
			TreePath path;
			TreeViewColumn column;
			this.treemap.GetCursor(out path, out column);
			TreeIter iter2 = new TreeIter();
			bool result = this.store.GetIter(out iter2, path);
			if (result)
			{
				this.store.Remove(ref iter2);
				if (!this.buttonSave.Sensitive)
				{
					this.buttonSave.Sensitive = true;
					this.buttonSend.Sensitive = false;
				}
			}
		}
		
		/// <summary>
		/// Function update edited link. Raised when user end edititing link.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> unused.
		/// </param>
		/// <param name="args">
		/// A <see cref="EditedArgs"/> path and new value for changed link.
		/// </param>
		void OnCellEdit(object sender, EditedArgs args)
		{
			TreeIter iter2 = new TreeIter();
			TreePath path = new TreePath(args.Path);
			bool result = this.store.GetIter(out iter2, path);
			if (result)
			{
				if (args.NewText != (string)this.store.GetValue(iter2, 0))
				{
					if (!System.Text.RegularExpressions.Regex.IsMatch(args.NewText, 
					                                                  @"^http(s?)\:\/\/[a-z0-9]*([a-z0-9\=\-\.\?\,\'\/\\\+&amp;%\$#_~]*)?$",
					                                                  System.Text.RegularExpressions.RegexOptions.IgnoreCase))
					{
						MessageDialog mesinvalid = new MessageDialog (this, 
						                                              DialogFlags.DestroyWithParent,
						                                              MessageType.Error, 
						                                              ButtonsType.Close, 
						                                              Catalog.GetString("Invalid url"));
						int result2 = mesinvalid.Run();
						if (result2 < 0)
						{
							mesinvalid.Destroy();
							mesinvalid.Dispose();
						}
						return;
					}
					this.store.SetValue(iter2, 0, args.NewText);
					if (!this.buttonSave.Sensitive)
					{
						this.buttonSave.Sensitive = true;
						this.buttonSend.Sensitive = false;
					}
				}
			}
		}

		/// <summary>
		/// Function enable btndelete if disabled. Raised when row selected in treemap.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> unused.
		/// </param>
		/// <param name="e">
		/// A <see cref="System.EventArgs"/> unused.
		/// </param>
		protected virtual void OnTreemapCursorChanged (object sender, System.EventArgs e)
		{
			if (!this.btndelete.Sensitive)
			{
				this.btndelete.Sensitive = true;
			}
		}
	}
}
