// SitemapURLDialog.cs
// 
// Copyright (C) 2008,2009,2010 Bartek Jasicki
//
// This file is part of Grubng.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

/**@file SitemapURLDialog.cs
 * Provide GTK Dialog for start create sitemap.
 * @fn virtual void Grubng::SitemapURLDialog::Build()
 * Create part of GUI and show it to user*/

using System;
using Gtk;
using Mono.Unix;

namespace Grubng
{	
	/// <summary>
	/// Provide GTK Dialog for start create sitemap.
	/// </summary>
	public partial class SitemapURLDialog : Dialog
	{
		/// <summary>
		/// GTK Button for exit from dialog.
		/// </summary>
		private Button buttonCancel;
		/// <summary>
		/// GTK Button for start create sitemap.
		/// </summary>
		private Button buttonOk;
		
		/// <summary>
		/// Standard class constructor. Create dialog and show it to user.
		/// </summary>
		public SitemapURLDialog()
		{
			this.Build();
			this.rbtnurl.Active = true;
			//Add button buttonCancel
			this.buttonCancel = new Button();
			this.buttonCancel.Image = new Image(System.IO.Path.Combine(System.AppDomain.CurrentDomain.BaseDirectory, 
			                                                           "Images" + System.IO.Path.DirectorySeparatorChar + 
			                                                           "cancel.png"));
			this.buttonCancel.ImagePosition = PositionType.Left;
			this.buttonCancel.Label = Catalog.GetString("_Cancel");
			this.buttonCancel.UseUnderline = true;
			this.AddActionWidget(buttonCancel, -6);
			this.buttonCancel.ShowAll();
			//Add button buttonOk 
			this.buttonOk = new Button();
			this.buttonOk.Image = new Image(System.IO.Path.Combine(System.AppDomain.CurrentDomain.BaseDirectory, 
			                                                       "Images" + System.IO.Path.DirectorySeparatorChar + 
			                                                       "ok.png"));
			this.buttonOk.ImagePosition = PositionType.Left;
			this.buttonOk.Label = Catalog.GetString("_OK");
			this.buttonOk.UseUnderline = true;
			this.AddActionWidget(buttonOk, -5);
			this.buttonOk.Clicked += new EventHandler(OnButtonOkClicked);
			this.buttonOk.ShowAll();
			this.edtsitemapurl.GrabFocus();
			//Add dialog icon
			this.Icon = new Gdk.Pixbuf(System.IO.Path.Combine(System.AppDomain.CurrentDomain.BaseDirectory, 
			                                                  "Images" + System.IO.Path.DirectorySeparatorChar + 
			                                                  "grubby.ico"));
		}
		
		/// <summary>
		/// Function create new sitemap.
		/// Raised when buttonOk clicked.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> Unused
		/// </param>
		/// <param name="e">
		/// A <see cref="System.EventArgs"/> Unused
		/// </param>
		private void OnButtonOkClicked (object sender, System.EventArgs e)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (e != null)
			{
				e = null;
			}
			int result;
			Config conf = new Config();
			string filepath = Config.ReadConfig("/configuration/work/workdir", conf.FilePath);
			conf = null;
			Guid guid = Guid.NewGuid();
			string filename;
			if (this.rbtnurl.Active)
			{
				//Check correctness of inserted url
				if (!System.Text.RegularExpressions.Regex.IsMatch(this.edtsitemapurl.Text, 
				                                                  @"^http(s?)\:\/\/[a-z0-9]*([a-z0-9\=\-\.\?\,\'\/\\\+&amp;%\$#_~]*)?$",
				                                                  System.Text.RegularExpressions.RegexOptions.IgnoreCase))
				{
					MessageDialog mesinvalid = new MessageDialog (this, 
					                                              DialogFlags.DestroyWithParent,
					                                              MessageType.Error, 
					                                              ButtonsType.Close, 
					                                              Catalog.GetString("Invalid url"));
					result = mesinvalid.Run();
					if (result < 0)
					{
						mesinvalid.Destroy();
						mesinvalid.Dispose();
					}
					return;
				}
				//Create sitemap - fetch page and create xml file
				this.edtsitemapurl.Sensitive = false;
				this.buttonOk.Sensitive = false;
				//Downloading page
				this.lblstatus.Text = Catalog.GetString("Status:") + " " + Catalog.GetString("Downloading page");
				while (Application.EventsPending())
				{
					Application.RunIteration();
				}
				filename = filepath + guid.ToString() + ".arc";
				string[] hostname = this.edtsitemapurl.Text.Split('/');
				Crawler grubng = new Crawler();
				grubng.SetPath();
				System.Collections.Generic.List<string> headers = new System.Collections.Generic.List<string>();
				headers.Add("User-Agent: GrubNG");
				headers.Add("Accept: text/html");
				System.IO.FileStream workunit = System.IO.File.Create(filename);
				System.IO.FileStream tmpsitemap = System.IO.File.Create(filename + ".tmp");
				int[] httpresult = grubng.Crawling2(this.edtsitemapurl.Text, hostname[2], ref workunit, "HTTP/1.0", headers, 
				                                    ref tmpsitemap, true);
				workunit.Close();
				workunit = null;
				tmpsitemap.Close();
				tmpsitemap = null;
				grubng = null;
				System.IO.File.Delete(filename);
				if (httpresult[0] != 200)
				{
					this.lblstatus.Text = Catalog.GetString("Status:") + " " + 
						Catalog.GetString("Error during downloading page. HTTP answer code:") + " " + 
							httpresult[0].ToString();
					this.edtsitemapurl.Sensitive = true;
					this.buttonOk.Sensitive = true;
					return;
				}
				//Generate sitemap from temporary file
				this.lblstatus.Text = Catalog.GetString("Status:") + " " + Catalog.GetString("Generate sitemap");
				while (Application.EventsPending())
				{
					Application.RunIteration();
				}
				filename = filepath + guid.ToString() + ".arc.xml";
				Sitemap sitemap = new Sitemap(filename);
				bool result2 = sitemap.CreateSitemap(false);
				sitemap = null;
				if (!result2)
				{
					this.lblstatus.Text = Catalog.GetString("Status:") + " " +
						Catalog.GetString("Cannot generate sitemap for selected page.");
					this.edtsitemapurl.Sensitive = true;
					this.buttonOk.Sensitive = true;
					return;
				}
			}
			else
			{
				if (this.rbtnempty.Active)
				{
					filename = filepath + guid.ToString() + ".xml";
				}
				else
				{
					FileChooserDialog filedialog = new FileChooserDialog(Catalog.GetString("Select sitemap for upload"),
					                                                     this,
					                                                     FileChooserAction.Open,
					                                                     Catalog.GetString("Close"), ResponseType.Close,
					                                                     Catalog.GetString("Open"), ResponseType.Accept);
					FileFilter filter = new FileFilter();
					filter.Name = Catalog.GetString("Sitemap or Compressed sitemap");
					filter.AddMimeType("text/xml");
					filter.AddPattern("*.xml");
					filter.AddMimeType("application/x-gzip");
					filter.AddPattern("*.xml.gz");
					filedialog.AddFilter(filter);
					filedialog.SetCurrentFolder(filepath);
					result = filedialog.Run();
					filename = null;
					if (result == -3)
					{
						filename = filedialog.Filename;
					}
					filedialog.Destroy();
					filedialog.Dispose();
					if (String.IsNullOrEmpty(filename))
					{
						return;
					}
					this.lblstatus.Text = Catalog.GetString("Status:") + " " + Catalog.GetString("Checking sitemap");
					Sitemap sitemap = new Sitemap(filename);
					if (!sitemap.CheckSitemap())
					{
						MessageDialog mesinvalid = new MessageDialog (this, 
						                                              DialogFlags.DestroyWithParent,
						                                              MessageType.Error, 
						                                              ButtonsType.Close, 
						                                              Catalog.GetString("Invalid sitemap file"));
						result = mesinvalid.Run();
						if (result < 0)
						{
							mesinvalid.Destroy();
							mesinvalid.Dispose();
						}
						return;
					}
					sitemap = null;
				}
			}
			this.Hide();
			ShowSitemapDialog sitemapdlg = new ShowSitemapDialog(filename);
			result = 0;
			while (result != -6)
			{
				result = sitemapdlg.Run();
				if (result == -4)
				{
					result = -6;
				}
			}
			sitemapdlg.Destroy();
			sitemapdlg.Dispose();
			this.buttonCancel.Click();
		}

		/// <summary>
		/// Function enable or disable url field.
		/// Raised when rbtnurl toggled.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> Unused
		/// </param>
		/// <param name="e">
		/// A <see cref="System.EventArgs"/> Unused
		/// </param>
		protected virtual void OnRbtnurlToggled (object sender, System.EventArgs e)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (e != null)
			{
				e = null;
			}
			this.label1.Sensitive = this.rbtnurl.Active;
			this.edtsitemapurl.Sensitive = this.rbtnurl.Active;
		}
	}
}
