// SplashScreen.cs
// 
// Copyright (C) 2008,2010 Bartek Jasicki
//
// This file is part of Grubng.
//
//  Grubng is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//   Grubng is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with Grubng.  If not, see <http://www.gnu.org/licenses/>.
//

/**@file SplashScreen.cs
 * Provide GTK Window for splash screen.
 * @fn virtual void Grubng::SplashScreen::Build()
 * Create part of GUI and show it to user*/

using System;
using Gtk;

namespace Grubng
{
	
	/// <summary>
	/// Provide GTK Window for splash screen.
	/// </summary>
	internal sealed partial class SplashScreen : Gtk.Window
	{
		/// <summary>
		/// GTK Label to show current status of program.
		/// </summary>
		Label lblstatus;
		/// <summary>
		/// GTK Progress Bar to show current progress of program start.
		/// </summary>
		ProgressBar prgstatus;
		
		/// <summary>
		/// Standard class constructor.
		/// </summary>
		public SplashScreen() : base(Gtk.WindowType.Toplevel)
		{
			this.Build();
			this.lblstatus = new Label(Mono.Unix.Catalog.GetString("Start..."));
			this.vbox1.Add(this.lblstatus);
			Box.BoxChild w2 = ((Gtk.Box.BoxChild)(this.vbox1[this.lblstatus]));
            w2.Position = 1;
            w2.Expand = false;
            w2.Fill = false;
			this.lblstatus.Show();
			this.prgstatus = new ProgressBar();
			this.vbox1.Add(this.prgstatus);
			Box.BoxChild w3 = ((Gtk.Box.BoxChild)(this.vbox1[this.prgstatus]));
            w3.Position = 2;
            w3.Expand = false;
            w3.Fill = false;
			this.prgstatus.Show();
			this.imgsplash.Pixbuf = new Gdk.Pixbuf(System.IO.Path.Combine(System.AppDomain.CurrentDomain.BaseDirectory, "Images" + System.IO.Path.DirectorySeparatorChar + "splash.jpg"));
			while (Application.EventsPending())
			{
				Application.RunIteration();
			}
		}
		
		/// <summary>
		/// Property set private field lblstatus.
		/// </summary>
		public string LabelStatus
		{
			get {return this.lblstatus.Text;}
			set {this.lblstatus.Text = value;}
		}
		
		/// <summary>
		/// Property set private field prgstatus.
		/// </summary>
		public ProgressBar PrgStatus
		{
			get {return this.prgstatus;}
			set {this.prgstatus = value;}
		}
		
		/// <summary>
		/// Function override Gtk.Object.Dispose function.
		/// </summary>
		public override void Dispose()
		{
			this.vbox1.Dispose();
			this.imgsplash.Dispose();
			this.lblstatus.Dispose();
			this.prgstatus.Dispose();
			base.Dispose();
		}
	}
}
