// StartWinFoCatalog.cs
// 
// Copyright (C) 2008,2009,2010 Bartek Jasicki
//
// This file is part of Grubng.
//
//  Grubng is free software: you can redistribute it and/or modify
//  it under the teCatalogs of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//   Grubng is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with Grubng.  If not, see <http://www.gnu.org/licenses/>.
//

/**@file StartWinForm.cs
 * Start WinForm GUI of program.*/

using System;
using System.IO;
using System.Windows.Forms;
using System.Diagnostics;
using Mono.Unix;

namespace Grubng
{
	
	/// <summary>
	/// Start WinForm GUI of program.
	/// </summary>
	internal static class StartWinForm
	{
	
		/// <summary>
		/// Create WinForm GUI of program.
		/// </summary>
		public static void StartGUI()
		{
			Application.EnableVisualStyles();
			Application.SetCompatibleTextRenderingDefault(false);
			AppDomain.CurrentDomain.UnhandledException += new UnhandledExceptionEventHandler(WinUnhandledException);
			//Check mono version for compatibility
			if ((Environment.Version.Major == 1) && (Environment.Version.Minor == 2) && (Environment.Version.Build < 5))
			{
				WinMessageBox.Show(Catalog.GetString("Unfortunately, you have too old version of Mono to run program.") + 
				                   Catalog.GetString("You need at least Mono 1.2.6"));
				return;
			}
			//Check for running instance of client (if any, teCataloginate this and show info to user)
			string filepath = Environment.GetFolderPath(Environment.SpecialFolder.Personal) + 
				Path.DirectorySeparatorChar + ".grub" + Path.DirectorySeparatorChar;
			if (File.Exists(filepath + "grub.pid"))
			{
				WinMessageBox.Show(Catalog.GetString("You cannot run more than one client simultaneously"));
				return; 
			}
			bool checkfile = File.Exists(filepath + ".grub.xml");
			if (!checkfile)
			{
				Application.Run(new WinLogonDialog());
				checkfile = File.Exists(filepath + ".grub.xml");
				if (!checkfile)
				{
					return;
				}
			}
			//Create .pid file
			MainClass.CreatePIDFile();
			if (checkfile)
			{
				Application.Run(new WinSplashScreen());
				Application.Run(new WinMainWindow());
			}
		}
		
		/// <summary>
		/// Function catch unhandled exceptions for WinForm GUI. Write infoCatalogations about exception to error.log and show
		/// information dialog about error.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> unused
		/// </param>
		/// <param name="args">
		/// A <see cref="UnhandledExceptionEventArgs"/> provide infoCatalogations about error (source, stacktrace,
		/// general info about error)
		/// </param>
		static void WinUnhandledException(object sender, UnhandledExceptionEventArgs args)
		{
			MainClass.MainUnhandledException(sender, args);
            string message = Catalog.GetString("Critical error appear and client been closed.") + 
				Environment.NewLine +
					Catalog.GetString("Please report bug on http://grub.org.") +
					Environment.NewLine + 
					Catalog.GetString("To report add file error.log from your work directory.");
			WinMessageBox.Show(message);
		}
	}
}
