// WinAboutDialog.cs
// 
// Copyright (C) 2008,2011 Bartek Jasicki
//
// This file is part of Grubng.
//
//  Grubng is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//   Grubng is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with Grubng.  If not, see <http://www.gnu.org/licenses/>.
//

/**@file WinAboutDialog.cs
 * Provide WinForms Window for informations about program.
 * @fn void Grubng::WinAboutDialog::InitializeComponent()
 * Create part of GUI.*/

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Reflection;

namespace Grubng
{
	/// <summary>
	/// Provide WinForm Window for informations about program.
	/// </summary>
	internal sealed partial class WinAboutDialog : Form
	{
		/// <summary>
		/// Standard class constructor. Create GUI and show it to user.
		/// </summary>
		public WinAboutDialog()
		{
			InitializeComponent();
			Assembly asm = Assembly.GetExecutingAssembly();
			object[] attributes = asm.GetCustomAttributes(typeof(AssemblyCopyrightAttribute), false);
			this.lblversion.Text = asm.GetName().Name + " " + asm.GetName().Version.ToString();
			this.lblcopy.Text = ((AssemblyCopyrightAttribute)attributes[0]).Copyright;
		}
		
		/// <summary>
		/// Function show license text to user. Raised when btnlicense clicked.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> Unused.
		/// </param>
		/// <param name="e">
		/// A <see cref="EventArgs"/> Unused.
		/// </param>
		static void OnLicense (object sender, EventArgs e)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (e != null)
			{
				e = null;
			}
			WinHelpDialog licensedlg = new WinHelpDialog("COPYING");
			DialogResult result = licensedlg.ShowDialog();
			if (result != DialogResult.None)
			{
				licensedlg.Dispose();
			}
		}
		
		/// <summary>
		/// Function show authors list to user. Raised when btnauthors clicked.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> Unused.
		/// </param>
		/// <param name="e">
		/// A <see cref="EventArgs"/> Unused.
		/// </param>
		static void OnAuthors (object sender, EventArgs e)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (e != null)
			{
				e = null;
			}
			WinHelpDialog authorsdlg = new WinHelpDialog("AUTHORS");
			DialogResult result = authorsdlg.ShowDialog();
			if (result != DialogResult.None)
			{
				authorsdlg.Dispose();
			}
		}
		
		/// <summary>
		/// Function open web browser on grub page. Raised when btnurl clicked.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> Unused.
		/// </param>
		/// <param name="e">
		/// A <see cref="EventArgs"/> Unused.
		/// </param>
		static void OnHomePage (object sender, EventArgs e)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (e != null)
			{
				e = null;
			}
			System.Diagnostics.Process proc = System.Diagnostics.Process.Start("http://grub.org");
			proc.Close();
		}
		
		/// <summary>
		/// Function close window. Raised when btnok clicked.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> Unused.
		/// </param>
		/// <param name="e">
		/// A <see cref="EventArgs"/> Unused.
		/// </param>
		void OnBtnOkClick (object sender, EventArgs e)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (e != null)
			{
				e = null;
			}
			this.Close();
		}
		
		/// <summary>
		/// Function override System.ComponentModel.Component.Dispose function.
		/// </summary>
		public new void Dispose()
		{
			this.btnauthors.Dispose();
			this.btnlicense.Dispose();
			this.btnok.Dispose();
			this.btnurl.Dispose();
			this.lblcopy.Dispose();
			this.lblinfo.Dispose();
			this.lblversion.Dispose();
			this.pclogo.Dispose();
			base.Dispose();
		}
	}
}
