// WinCrawlerInfo.cs
// 
// Copyright (C) 2008,2009,2010 Bartek Jasicki
//
// This file is part of Grubng.
//
//  Grubng is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//   Grubng is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with Grubng.  If not, see <http://www.gnu.org/licenses/>.
//

/**@file WinCrawlerInfo.cs
 * Provide WinForm Window for show informations about selected crawler.
 * @fn virtual void Grubng::WinCrawlerInfo::InitializeComponent()
 * Create GUI*/

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Timers;
using System.Threading;
using Mono.Unix;

namespace Grubng
{
	/// <summary>
	/// Provide WinForm Window for show informations about selected crawler.
	/// </summary>
	internal sealed partial class WinCrawlerInfo : Form
	{
		/// <summary>
		/// Number of crawler;
		/// </summary>
		int num;
		/// <summary>
		/// Thread of crawler
		/// </summary>
		Threads crawlerthread;
		/// <summary>
		/// Status of crawler
		/// </summary>
		int status;
		/// <summary>
		/// Array of translated crawler statuses
		/// </summary>
		string[] crawlerstatus;
		/// <summary>
		/// Array of translated crawler messages
		/// </summary>
		string[] crawlermessage;
		
		/// <summary>
		/// Standatd class constructor. Create GUI and show it to user.
		/// </summary>
		/// <param name="number">
		/// A <see cref="System.Int32"/> number of crawler
		/// </param>
		/// <param name="thread">
		/// A <see cref="Threads"/> thread of crawler
		/// </param>
		public WinCrawlerInfo(int number, Threads thread)
		{
			InitializeComponent();
			this.crawlerstatus = new string[11]{Catalog.GetString("Stopped"),
				Catalog.GetString("Paused"),
				Catalog.GetString("In Queue"),
				Catalog.GetString("Downloading"),
				Catalog.GetString("Download Error"),
				Catalog.GetString("Crawling"),
				Catalog.GetString("Uploading"),
				Catalog.GetString("Upload Error"),
				Catalog.GetString("Waiting"),
				Catalog.GetString("Invalid arc file"),
				Catalog.GetString("Program error")
			};
			this.crawlermessage = new string[24]{Catalog.GetString("Waiting in queue for download new workunit"),
				Catalog.GetString("Waiting in queue for upload workunit"),
				Catalog.GetString("Invalid login. Check your setting for username and password."),
				Catalog.GetString("Request timeout."),
				Catalog.GetString("Problems with connection to Internet."),
				Catalog.GetString("Unknown error"),
				Catalog.GetString("Done"),
				Catalog.GetString("File checked and compressed, ready to upload."),
				Catalog.GetString("Workunit uploaded"),
				Catalog.GetString("Invalid workunit, you must try crawl it again"),
				Catalog.GetString("Timeout reached when uploading. Please check your internet connection"),
				Catalog.GetString("Cannot connect to server. Try upload workunit later."),
				Catalog.GetString("Internal Server Error. Try download workunit later."),
				Catalog.GetString("No server response. Try upload workunit again."),
				Catalog.GetString("Program error appear and crawler was stopped."),
				Catalog.GetString("Internal Server Error. Try upload .arc file later."),
				Catalog.GetString("Sitemap uploaded."),
				Catalog.GetString("Internal Server Error. Sitemap cannot be uploaded."),
				Catalog.GetString("Generate sitemap."),
				Catalog.GetString("Upload .arc files to this server is temporary disabled. Please try later."),
				Catalog.GetString("You was sent this file earlier. You cannot send this same .arc file twice."),
				Catalog.GetString("Someone was sent this same URL's earlier."),
				Catalog.GetString("No workunits available at this moment. Please try later."),
				Catalog.GetString("Upload sitemaps to this server is disabled.")
			};
			this.num = number;
			this.status = -1;
			this.crawlerthread = thread;
			this.crawlerthread.messageEmit += new EventHandler(this.OnMessage);
			this.Activated += new EventHandler(this.OnActivate);
			this.Activate();
		}
		
		/// <summary>
		/// Function show current status of crawler. Raised when window is activated.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> unused
		/// </param>
		/// <param name="e">
		/// A <see cref="EventArgs"/> unused
		/// </param>
		private void OnActivate(object sender, EventArgs e)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (e != null)
			{
				e = null;
			}
			this.crawlerthread.OnMessageEmit(EventArgs.Empty);
			this.Activated -= new EventHandler(this.OnActivate);
		}
		
		/// <summary>
		/// Function close window. Raised when btnok clicked.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> unused
		/// </param>
		/// <param name="e">
		/// A <see cref="EventArgs"/> unused
		/// </param>
		void OnOkClick(object sender, EventArgs e)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (e != null)
			{
				e = null;
			}
			this.crawlerthread.messageEmit -= new EventHandler(this.OnMessage);
			this.Close();
		}
		
		/// <summary>
		/// Function clear txtinfo textbox. Raised when btnclear clicked.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> unused
		/// </param>
		/// <param name="e">
		/// A <see cref="EventArgs"/> unused
		/// </param>
		void OnClearClick(object sender, EventArgs e)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (e != null)
			{
				e = null;
			}
			this.txtinfo.Clear();
		}
		
		delegate void DelegateUpdateText(string text);
		
		/// <summary>
		/// Function update GUI. Raised when crawler thread emits message event.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> unused
		/// </param>
		/// <param name="e">
		/// A <see cref="EventArgs"/> unused
		/// </param>
		void OnMessage(object sender, EventArgs e)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (e != null)
			{
				e = null;
			}
			string tdate = DateTime.Now.ToString("HH:mm:ss");
			int crawlstatus = this.crawlerthread.GetField(this.num, "status");
			int message = this.crawlerthread.GetField(this.num, "message");
			string servermessage = this.crawlerthread.GetStringField(this.num, "servermessage");
			if (this.status != crawlstatus)
			{
				this.Invoke(new DelegateUpdateText(UpdateText), tdate + " " + this.crawlerstatus[crawlstatus] + Environment.NewLine);
				this.status = crawlstatus;
			}
			if (message != -1)
			{
				this.Invoke(new DelegateUpdateText(UpdateText), tdate + " " + this.crawlermessage[message] + Environment.NewLine);
				this.crawlerthread.SetField(this.num, "message", 0);
			}
			if (servermessage.Length > 0)
			{
				this.Invoke(new DelegateUpdateText(UpdateText), tdate + " " + Catalog.GetString("Server response:") + " " +
				            servermessage + Environment.NewLine);
				this.crawlerthread.SetField(this.num, "servermessage", 0);
			}
			if (this.status == 5)
			{
				string cururl = this.crawlerthread.GetStringField(this.num, "cururl");
				int curstatus = this.crawlerthread.GetField(this.num, "curstatus");
				if (cururl.Length > 0)
				{
					this.Invoke(new DelegateUpdateText(UpdateText), tdate + 
					            " [" + this.crawlerthread.GetField(this.num, "curiter").ToString() + 
							"/" + (this.crawlerthread.GetField(this.num, "iter") - 1).ToString() + "] " + cururl + " ");
					this.crawlerthread.SetField(this.num, "cururl", 0);
				}
				if (curstatus != 0)
				{
					this.Invoke(new DelegateUpdateText(UpdateText), Catalog.GetString("[Status:") + " " + curstatus.ToString() + "]" + 
					            Environment.NewLine);
					this.crawlerthread.SetField(this.num, "curstatus", 0);
				}
			}
		}
		
		/// <summary>
		/// Function update txtinfo from other thread.
		/// </summary>
		/// <param name="text">
		/// A <see cref="System.String"/> text to add to txtinfo.
		/// </param>
		void UpdateText(string text)
		{
			this.txtinfo.Text += text;
		}
		
		/// <summary>
		/// Move cursor in console to end. Raised when text inserted to txtinfo.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> unused
		/// </param>
		/// <param name="args">
		/// A <see cref="EventArgs"/> unused
		/// </param>
		void OnTextChanged(object sender, EventArgs args)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (args != null)
			{
				args = null;
			}
			if (this.txtinfo.Text.Length > 0)
			{
				this.txtinfo.SelectionStart = this.txtinfo.Text.Length - 1;
				this.txtinfo.ScrollToCaret();
			}
		}
		
		/// <summary>
		/// Function resize buttons and richtextbox. Raised when window is resized.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> unused
		/// </param>
		/// <param name="args">
		/// A <see cref="EventArgs"/> unused
		/// </param>
		void OnResize(object sender, EventArgs args)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (args != null)
			{
				args = null;
			}
			int x = this.Size.Width - 490;
			int y = this.Size.Height - 414;
			this.btnclear.Location = new Point(312 + x, 352 + y);
			this.btnok.Location = new Point(400 + x, 352 + y);
			this.txtinfo.Size = new Size(480 + x, 344 + y);
		}
		
		/// <summary>
		/// Function show selected link in browser. Raised when mouse click on txtinfo.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> unused
		/// </param>
		/// <param name="args">
		/// A <see cref="MouseEventArgs"/> coordinates of mouse click
		/// </param>
		void OnTextClick(object sender, MouseEventArgs args)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (this.txtinfo.Lines.Length == 0)
			{
				return;
			}
			System.Text.RegularExpressions.Regex rx = new System.Text.RegularExpressions.Regex(@"^(ht|f)tp(s?)\:\/\/[0-9a-zA-Z]([-.\w]*[0-9a-zA-Z])*(:(0-9)*)*(\/?)([a-zA-Z0-9\-\.\?\,\'\/\\\+&amp;%\$#_~]*)?$");
			Point pt = args.Location;
			int index = this.txtinfo.GetCharIndexFromPosition(pt);
			int line = this.txtinfo.GetLineFromCharIndex(index);
			if (line >= this.txtinfo.Lines.Length)
			{
				line = this.txtinfo.Lines.Length - 1;
			}
			string text = this.txtinfo.Lines[line];
			string[] splitline = text.Split(' ');
			if (splitline.Length < 3)
			{
				return;
			}
			if (!rx.IsMatch(splitline[2]))
			{
				return;
			}
			rx = null;
			System.Diagnostics.Process proc = System.Diagnostics.Process.Start(splitline[2]);
			proc.Close();
			text = null;
			splitline = null;
		}
		
		/// <summary>
		/// Function override System.ComponentModel.Component.Dispose function.
		/// </summary>
		public new void Dispose()
		{
			this.btnok.Dispose();
			this.btnclear.Dispose();
			this.txtinfo.Dispose();
			base.Dispose();
		}
	}
}
