// WinHelpDialog.cs
// 
// Copyright (C) 2008 Bartek Jasicki
//
// This file is part of Grubng.
//
//  Grubng is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//   Grubng is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with Grubng.  If not, see <http://www.gnu.org/licenses/>.
//

/**@file WinHelpDialog.cs
 * Provide WinForms Window for show text file
 * @fn void Grubng::WinHelpDialog::InitializeComponent()
 * Create part of GUI.*/

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Text;
using System.Windows.Forms;

namespace Grubng
{
	/**Provide WinForm Window for show text file.*/
	internal sealed partial class WinHelpDialog : Form
	{
		/**Class contructor, create dialog and show it to user.
		 * @param filename contain name of text file to show*/
		public WinHelpDialog(string filename)
		{
			InitializeComponent();
			this.Text = filename;
			//Read README.txt
			FileStream file = System.IO.File.OpenRead(System.IO.Path.Combine(System.AppDomain.CurrentDomain.BaseDirectory, filename));
			StreamReader stream = new StreamReader(file);
			txthelp.Text = stream.ReadToEnd();
			stream.Close();
			file.Close();
		}
		
		/**Destroying help dialog.
		 * @param sender Unused
		 * @param e Unused*/
		private void OnBtnOkClick(object sender, EventArgs e)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (e != null)
			{
				e = null;
			}
			this.Close();
		}
		
		/**Function override System.ComponentModel.Component.Dispose function.*/
		public new void Dispose()
		{
			this.btnok.Dispose();
			this.txthelp.Dispose();
			base.Dispose();
		}
	}
}
