// WinLogonDialog.cs
// 
// Copyright (C) 2008.2009,2010 Bartek Jasicki
//
// This file is part of Grubng.
//
//  Grubng is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//   Grubng is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with Grubng.  If not, see <http://www.gnu.org/licenses/>.
//

/**@file WinLogonDialog.cs
 * Provide WinForms Dialog for set username and password
 * @fn void Grubng::WinLogonDialog::InitializeComponent()
 * Create part of GUI and show it to user*/

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Mono.Unix;

namespace Grubng
{
	/// <summary>
	/// Provide WinForms Dialog for set username and password.
	/// </summary>
	internal sealed partial class WinLogonDialog : Form
	{
		
		/// <summary>
		/// Standard class constructor. Create dialog and show it to user.
		/// </summary>
		public WinLogonDialog()
		{
			InitializeComponent();
			this.Text = Catalog.GetString("Logon");
			this.Icon = new Icon(System.IO.Path.Combine(System.AppDomain.CurrentDomain.BaseDirectory, "Images" + System.IO.Path.DirectorySeparatorChar + "grubby.ico"));
			this.btncancel.Text = Catalog.GetString("Cancel");
			this.btncancel.Click += new EventHandler(OnBtnCancel);
			this.btnok.Text = Catalog.GetString("OK");
			this.btnok.Click += new EventHandler(OnBtnOk);
			this.btnregister.Text = Catalog.GetString("Create new Grub account");
			this.btnregister.Click += new EventHandler(OnBtnRegister);
			this.lbluser.Text = Catalog.GetString("Username:");
			this.lblpass.Text = Catalog.GetString("Password:");
		}
		
		/// <summary>
		/// Function close window. Raised when button btncancel clicked.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> unused.
		/// </param>
		/// <param name="e">
		/// A <see cref="EventArgs"/> unused.
		/// </param>
		void OnBtnCancel(object sender, EventArgs e)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (e != null)
			{
				e = null;
			}
			this.Close();
		}
		
		/// <summary>
		/// Function for create new grub.xml file.
		/// Function check for empty fields in form. If all is ok, create
		/// new grub.xml file. If not - show info about it.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> unused.
		/// </param>
		/// <param name="e">
		/// A <see cref="EventArgs"/> unused.
		/// </param>
		void OnBtnOk(object sender, EventArgs e)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (e != null)
			{
				e = null;
			}
			//Check for empty fields
			bool checkuser = String.IsNullOrEmpty(edtuser.Text);
			bool checkpass = String.IsNullOrEmpty(edtpass.Text);
			if (checkuser || checkpass)
			{
				WinMessageBox.Show(Catalog.GetString("Fill empty fields"));
			}
			else
			{
				Config conf = new Config();
				conf.CreateConfig(edtuser.Text, edtpass.Text);
				this.Close();
			}
		}
		
		/// <summary>
		/// Function open web browser on Grub create account page.
		/// Raised when btnregister clicked.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> unused.
		/// </param>
		/// <param name="e">
		/// A <see cref="EventArgs"/> unused.
		/// </param>
		static void OnBtnRegister(object sender, EventArgs e)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (e != null)
			{
				e = null;
			}
			System.Diagnostics.Process proc = System.Diagnostics.Process.Start("http://grub.org/?q=en/user/register");
			proc.Close();
		}
		
		/// <summary>
		/// Function override System.ComponentModel.Component.Dispose function.
		/// </summary>
		public new void Dispose()
		{
			this.lblpass.Dispose();
			this.lbluser.Dispose();
			this.edtpass.Dispose();
			this.edtuser.Dispose();
			this.btnregister.Dispose();
			this.btncancel.Dispose();
			this.btnok.Dispose();
			base.Dispose();
		}
	}
}
