// WinMessageBox.cs
// 
// Copyright (C) 2008 Bartek Jasicki
//
// This file is part of Grubng.
//
// Grubng is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Grubng is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Grubng.  If not, see <http://www.gnu.org/licenses/>.
//

/**@file WinAboutDialog.cs
 * Provide simple Message Box for program.*/

using System.Globalization;
using System.Windows.Forms;

namespace Grubng
{
	/**Provide simple Message Box for program.*/
	internal static class WinMessageBox
	{
		/**Show Message Box with informations about error.
		 * Function overload standard MessageBox.Show() function
		 * @param info Text to display in MessageBox*/
		public static void Show(string info)
		{
			MessageBoxOptions options = 0;
			if (CultureInfo.CurrentUICulture.TextInfo.IsRightToLeft)
			{
				options |= MessageBoxOptions.RtlReading | MessageBoxOptions.RightAlign;
			}
        
			DialogResult result = MessageBox.Show(info, 
			                                      System.String.Empty,
			                                      MessageBoxButtons.OK, 
			                                      MessageBoxIcon.Error, 
			                                      MessageBoxDefaultButton.Button1, 
			                                      options);
			if (result != DialogResult.None)
			{
				result = DialogResult.OK;
			}
		}
	}
}
