// WinSitemapURLDialog.cs
// 
// Copyright (C) 2008,2010 Bartek Jasicki
//
// This file is part of Grubng.
//
//  Grubng is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//   Grubng is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with Grubng.  If not, see <http://www.gnu.org/licenses/>.
//

/**@file WinSitemapURLDialog.cs
 * Provide WinForm window for create/edit/send sitemap.
 * @fn virtual void Grubng::WinSitemapURLDialog::InitializeComponent()
 * Create GUI*/


using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Mono.Unix;

namespace Grubng
{
	/// <summary>
	/// Provide WinForm window for create/edit/send sitemap.
	/// </summary>
	public partial class WinSitemapURLDialog : Form
	{
		
		/// <summary>
		/// Standard class constructor. Create GUI.
		/// </summary>
		public WinSitemapURLDialog()
		{
			InitializeComponent();
		}
		
		/// <summary>
		/// Function create new sitemap.
		/// Raised when buttonOk clicked.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> Unused
		/// </param>
		/// <param name="args">
		/// A <see cref="EventArgs"/> Unused
		/// </param>
		private void OnBtnOkClicked(object sender, EventArgs args)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (args != null)
			{
				args = null;
			}
			Config conf = new Config();
			string filepath = Config.ReadConfig("/configuration/work/workdir", conf.FilePath);
			conf = null;
			Guid guid = Guid.NewGuid();
			string filename = String.Empty;
			if (this.rbtncreate.Checked)
			{
				//Check correctness of inserted url
				if (!System.Text.RegularExpressions.Regex.IsMatch(this.edturl.Text, 
				                                                  @"^http(s?)\:\/\/[a-z0-9]*([a-z0-9\=\-\.\?\,\'\/\\\+&amp;%\$#_~]*)?$",
				                                                  System.Text.RegularExpressions.RegexOptions.IgnoreCase))
				{
					WinMessageBox.Show(Catalog.GetString("Invalid url"));
					return;
				}
				//Create sitemap - fetch page and create xml file
				this.edturl.Enabled = false;
				this.btnok.Enabled = false;
				//Downloading page
				this.lblstatus.Text = Catalog.GetString("Status:") + " " + Catalog.GetString("Downloading page");
				this.lblstatus.Update();
				filename = filepath + guid.ToString() + ".arc";
				string[] hostname = this.edturl.Text.Split('/');
				Crawler grubng = new Crawler();
				grubng.SetPath();
				System.Collections.Generic.List<string> headers = new System.Collections.Generic.List<string>();
				headers.Add("User-Agent: GrubNG");
				headers.Add("Accept: text/html");
				System.IO.FileStream workunit = System.IO.File.Create(filename);
				System.IO.FileStream tmpsitemap = System.IO.File.Create(filename + ".tmp");
				int[] httpresult = grubng.Crawling2(this.edturl.Text, hostname[2], ref workunit, "HTTP/1.0", headers, ref tmpsitemap,
				                                    true);
				workunit.Close();
				workunit = null;
				tmpsitemap.Close();
				tmpsitemap = null;
				grubng = null;
				System.IO.File.Delete(filename);
				if (httpresult[0] != 200)
				{
					this.lblstatus.Text = Catalog.GetString("Status:") + " " + 
						Catalog.GetString("Error during downloading page. HTTP answer code:") + " " + 
							httpresult[0].ToString();
					this.lblstatus.Update();
					this.edturl.Enabled = true;
					this.btnok.Enabled = true;
					return;
				}
				//Generate sitemap from temporary file
				this.lblstatus.Text = Catalog.GetString("Status:") + " " +
					Catalog.GetString("Generate sitemap");
				this.lblstatus.Update();
				filename = filepath + guid.ToString() + ".arc.xml";
				Sitemap sitemap = new Sitemap(filename);
				bool result = sitemap.CreateSitemap(false);
				sitemap = null;
				if (!result)
				{
					this.lblstatus.Text = Catalog.GetString("Status:") + " " +
						Catalog.GetString("Cannot generate sitemap for selected page.");
					this.lblstatus.Update();
					this.edturl.Enabled = true;
					this.btnok.Enabled = true;
					return;
				}
			}
			else
			{
				if (this.rbtnempty.Checked)
				{
					filename = filepath + guid.ToString() + ".xml";
				}
				else
				{
					OpenFileDialog filedlg = new OpenFileDialog();
					filedlg.Filter = Catalog.GetString("Sitemap or Compressed sitemap") + "|*.xml;*.xml.gz";
					filedlg.Title = Catalog.GetString("Upload sitemap on server");
					filedlg.InitialDirectory = filepath;
					DialogResult result = filedlg.ShowDialog();
					if (result == DialogResult.OK)
					{
						filename = filedlg.FileName;
					}
					filedlg.Dispose();
					if (String.IsNullOrEmpty(filename))
					{
						return;
					}
					this.lblstatus.Text = Catalog.GetString("Status:") + " " + 
						Catalog.GetString("Checking sitemap");
					Sitemap sitemap = new Sitemap(filename);
					if (!sitemap.CheckSitemap())
					{
						WinMessageBox.Show(Catalog.GetString("Invalid sitemap file"));
						return;
					}
					sitemap = null;
				}
			}
			this.Hide();
			WinShowSitemapDialog sitemapdlg = new WinShowSitemapDialog(filename);
			sitemapdlg.Show();
			this.OnBtnCancelClicked(null, null);
		}
		
		/// <summary>
		/// Function close window.
		/// Raised when btncancel clicked.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> Unused
		/// </param>
		/// <param name="args">
		/// A <see cref="EventArgs"/> Unused
		/// </param>
		private void OnBtnCancelClicked(object sender, EventArgs args)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (args != null)
			{
				args = null;
			}
			this.Close();
			this.Dispose();
		}
		
		/// <summary>
		/// Function enable or disable url field.
		/// Raised when rbtncreate toggled.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> Unused
		/// </param>
		/// <param name="args">
		/// A <see cref="EventArgs"/> Unused
		/// </param>
		private void OnRbtnCreateToggled(object sender, EventArgs args)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (args != null)
			{
				args = null;
			}
			this.label1.Enabled = this.rbtncreate.Checked;
			this.edturl.Enabled = this.rbtncreate.Checked;
		}
	}
}
