// WinSplashScreen.cs
// 
// Copyright (C) 2008,2009 Bartek Jasicki
//
// This file is part of Grubng.
//
//  Grubng is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//   Grubng is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with Grubng.  If not, see <http://www.gnu.org/licenses/>.
//

/**@file WinSplashScreen.cs
 * Provide WinForm Window for splash screen.
 * @fn void Grubng::WinSplashScreen::InitializeComponent()
 * Create part of GUI and show it to user*/

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Mono.Unix;

namespace Grubng
{
	/// <summary>
	/// Provide WinForm Window for splash screen.
	/// </summary>
	internal sealed partial class WinSplashScreen : Form
	{
		
		/// <summary>
		/// Standard class constructor.
		/// </summary>
		public WinSplashScreen()
		{
			InitializeComponent();
			this.pcbox.Image = new Bitmap(System.IO.Path.Combine(System.AppDomain.CurrentDomain.BaseDirectory, "Images" + System.IO.Path.DirectorySeparatorChar + "splash.jpg"));
			this.lblstatus.Text = Catalog.GetString("Start...");
			this.Activated += new EventHandler(OnActivate);
			this.Activate();
		}
		
		/// <summary>
		/// Function show splash screen and check for config files and updates. Raised when splash screen is activated.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> unused
		/// </param>
		/// <param name="e">
		/// A <see cref="EventArgs"/> unused
		/// </param>
		private void OnActivate(object sender, EventArgs e)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (e != null)
			{
				e = null;
			}
			Config conf = new Config();
			string runhidden = Config.ReadConfig("/configuration/other/runhidden", conf.FilePath);
			if (runhidden == "N")
			{
				this.Visible = true;
				this.pcbox.Update();
				this.lblstatus.Update();
				this.prgstatus.Update();
				this.Update();
				this.lblstatus.Text = Catalog.GetString("Checking correctness of configuration file...");
				this.lblstatus.Update();
				this.prgstatus.Value = 25;
				this.prgstatus.Update();
			}
			bool updateconf = conf.CheckConfig();
			if (runhidden == "N")
			{
				if (updateconf)
				{
					this.lblstatus.Text += Catalog.GetString("Updated");
					this.lblstatus.Update();
				}
				else
				{
					this.lblstatus.Text += Catalog.GetString("Done");
					this.lblstatus.Update();
				}
			}
			string checkupdates = Config.ReadConfig("/configuration/other/checkupdates", conf.FilePath);
			if (checkupdates == "Y")
			{
				string unstable = Config.ReadConfig("/configuration/other/unstable", conf.FilePath);
				if (runhidden == "N")
				{
					this.lblstatus.Text = Catalog.GetString("Checking for updates...");
					this.lblstatus.Update();
					this.prgstatus.Value = 50;
					this.prgstatus.Update();
				}
				System.Reflection.Assembly asm = System.Reflection.Assembly.GetExecutingAssembly();
				string version = asm.GetName().Version.Major.ToString() + "." + asm.GetName().Version.Minor.ToString() + "." + asm.GetName().Version.Build.ToString();
				Updater upd = new Updater(version);
				string update = upd.CheckUpdate(unstable);
				if ((update == "current") || (update == "older"))
				{
					if (runhidden == "N")
					{
						this.lblstatus.Text = Catalog.GetString("Updating...");
						this.lblstatus.Update();
						this.prgstatus.Value = 75;
						this.prgstatus.Update();
					}
					string[] updated = Updater.Update(update, unstable, String.Empty);
					string message = Catalog.GetString("Client successfully updated. ") + 
						Catalog.GetString("You must restart it again. ") + 
							Catalog.GetString("Click Ok button for exit.") + 
							Environment.NewLine + 
							Environment.NewLine + 
							Catalog.GetString("Updated files:") + 
							Environment.NewLine;
					for (int i = 0; i < updated.Length; i++)
					{
						message += (i + 1).ToString() + " " + updated[i] + Environment.NewLine;
					}
					WinMessageBox.Show(message);
					return;
				}
				conf = null;
				if (runhidden == "N")
				{
					if (update == "none")
					{
						this.lblstatus.Text += Catalog.GetString("none");
					}
					else
					{
						this.lblstatus.Text += Catalog.GetString("failed");
					}
					this.lblstatus.Update();
				}
			}
			if (runhidden == "N")
			{
				this.prgstatus.Value = 100;
				this.prgstatus.Update();
				System.Threading.Thread.Sleep(500);
			}
			this.Dispose();
		}
		
		/// <summary>
		/// Function override System.ComponentModel.Component.Dispose function.
		/// </summary>
		public new void Dispose()
		{
			this.prgstatus.Dispose();
			this.pcbox.Dispose();
			this.lblstatus.Dispose();
			base.Dispose();
		}
	}
}
